/*
 * NuevoAction.java		10-ago-2005
 */
package codetic.actions.admon.codigos.traducciones.secciones;

import javax.servlet.http.*;
import javax.sql.*;

import java.util.*;

import org.apache.struts.action.*;

import codetic.dto.*;
import codetic.bo.*;
import codetic.utils.*;


/**
 * Action que almacena todos los objetos necesarios para poder
 * crear una nueva traducción de sección. 
 * 
 * Puede realizar 3 posibles acciones, indicadas por el parámetro "accion":
 * 
 * 1. CREAR
 *    Almacena los objetos necesarios para la creación de una única traducción de la sección.
 * 
 * 2. COMENZAR
 *    Comienzo de una revisión de traducciones de sección sin traducir, dado un código y
 *    un idioma. Lo que hace es crear una pila con las secciones no traducidas.
 *    También almacena los objetos necesarios para la creación de una traducción de la sección.
 * 
 * 3. CONTINUAR
 *    Almacena los objetos necesarios para la creación de una traducción de la sección, utilizando
 *    la pila de secciones no traducidas creada en la acción COMENZAR.
 *    Esta acción no debe preocuparse de comprobar si la pila está vacía, ya que la página JSP asociada
 *    se encarga de no permitir que el usuario llame a este Action con la acción CONTINUAR una vez que la
 *    pila esté vacía.
 *
 * @author Miguel Angel Serna Mata
 */
public class NuevoAction extends Action{
	
	public ActionForward execute(ActionMapping mapping,
            ActionForm form,
			 HttpServletRequest request,
			 HttpServletResponse response)
    throws Exception {
		
	    ActionMessages messages = new ActionMessages();
		DataSource ds = null;
		String idAsociacion = null;
		String idCodigo = null;
		String idIdioma = null;
		String nivel = null;
		String accion = null;
		CodigoBO codigoBO = null;
		CodigoDTO codigo = null;
		GrupoDeTemasBO grupoDeTemasBO = null;
		Map map = new HashMap();
		int[] dividirEn2;
		SeccionBO seccionBO = null;
		SeccionDTO seccion = null;
		Stack pila = null;
		Integer totalSinTraducir = null;
		int numeroSeccion = 0;
		HttpSession session = null;
		AsociacionBO asociacionBO = null;
		AsociacionDTO asociacion = null;
		IdiomaBO idiomaBO = null;
		IdiomaDTO idioma = null;
		
        // Recuperar parámetros
		idAsociacion = request.getParameter(Ctes.ID_ASOCIACION);
		idCodigo = request.getParameter(Ctes.ID_CODIGO);
		idIdioma = request.getParameter(Ctes.ID_IDIOMA);
		accion = request.getParameter(Ctes.ACCION);
		    
		// Guardar el bean "parametros"
	    request.setAttribute("parametros",new String("&" + Ctes.ID_ASOCIACION + "=" + idAsociacion
	            + "&" + Ctes.ID_CODIGO + "=" + idCodigo
	    		+ "&" + Ctes.ID_IDIOMA + "=" + idIdioma));
	    
		// Guardar los parámetros
		request.setAttribute(Ctes.ID_ASOCIACION,idAsociacion);
		
		// Obtener el DataSource
		ds = getDataSource(request,Ctes.DATA_SOURCE);
		
		// Crear el Map(Grupo,{Temas})
		grupoDeTemasBO = new GrupoDeTemasBO(ds);
		map = grupoDeTemasBO.mapGruposYTemas();
		request.setAttribute(Ctes.MAP_GRUPOS_Y_TEMAS,map);
		
		// Obtener y establecer parámetros para poner los temas en 2 columnas
		dividirEn2 = grupoDeTemasBO.dividirMapEn2(map);
		request.setAttribute(Ctes.GRUPO_I,new Integer(dividirEn2[0]));
		request.setAttribute(Ctes.TEMA_J,new Integer(dividirEn2[1]));
		request.setAttribute(Ctes.NTEMAS,new Integer(dividirEn2[2]));
		
		// 1. Accion == CREAR
		// ------------------
		if (accion.equals(Ctes.CREAR)) {
		    // Recuperar la sesión
		    session = request.getSession();
		    
		    // Recuperar el nivel
		    nivel = request.getParameter(Ctes.NIVEL);
		    
		    // Recuperar la sección
			seccionBO = new SeccionBO(ds);
			seccion = seccionBO.recuperarSeccion(nivel,idCodigo);
			request.setAttribute(Ctes.SECCION,seccion);
			
			// Eliminamos paramámetros relacionados con las secciones no traducidas
			session.removeAttribute(Ctes.PILA_SECCIONES_SIN_TRADUCIR);
			session.removeAttribute(Ctes.TOTAL_SIN_TRADUCIR);
			session.removeAttribute(Ctes.NUMERO_SECCION);
		} else
		
		// 2. Accion == COMENZAR
		// ---------------------
		if (accion.equals(Ctes.COMENZAR)) {
		    // Recuperar la sesión
		    session = request.getSession();
		    
			// Recuperar las secciones no traducidas
			seccionBO = new SeccionBO(ds);
			pila = seccionBO.pilaSeccionesSinTraducir(idIdioma,idCodigo);
			
			// Recuperamos la primera sección de la pila
			seccion = (SeccionDTO) pila.pop();
			
			// Guardamos paramámetros relacionados con las secciones no traducidas
			session.setAttribute(Ctes.PILA_SECCIONES_SIN_TRADUCIR,pila);
			session.setAttribute(Ctes.TOTAL_SIN_TRADUCIR,new Integer(pila.size()+1));
			session.setAttribute(Ctes.NUMERO_SECCION,new Integer(1));
			request.setAttribute(Ctes.SECCION,seccion);
		} else
		
		// 3. Accion == CONTINUAR
		// ----------------------
		if (accion.equals(Ctes.CONTINUAR)) {
		    // Recuperar la sesión
		    session = request.getSession();
		    
			// Recuperar las secciones no traducidas
			seccionBO = new SeccionBO(ds);
			pila = (Stack)session.getAttribute(Ctes.PILA_SECCIONES_SIN_TRADUCIR);
			
			// Recuperamos la primera sección de la pila
			seccion = (SeccionDTO) pila.pop();
			
			// Recuperar parámetros
			totalSinTraducir = (Integer)session.getAttribute(Ctes.TOTAL_SIN_TRADUCIR);
			numeroSeccion = ((Integer)session.getAttribute(Ctes.NUMERO_SECCION)).intValue();
			
			// Guardamos paramámetros relacionados con las secciones no traducidas
			session.setAttribute(Ctes.PILA_SECCIONES_SIN_TRADUCIR,pila);
			session.setAttribute(Ctes.NUMERO_SECCION,new Integer(numeroSeccion+1));
			request.setAttribute(Ctes.SECCION,seccion);
		} else {
		    // Para cuando la validación falla
			// -------------------------------
		    // Recuperar el nivel
		    nivel = request.getParameter(Ctes.NIVEL);
		    
		    // Recuperar seccion
			seccionBO = new SeccionBO(ds);
			seccion = seccionBO.recuperarSeccion(nivel,idCodigo);
			request.setAttribute(Ctes.SECCION,seccion);
		}
		
		// Recuperar el código
	    codigoBO = new CodigoBO(ds);
		codigo = codigoBO.recuperarCodigo(idCodigo);
		request.setAttribute(Ctes.CODIGO,codigo);
		
		// Recuperar la asociación
		asociacionBO = new AsociacionBO(ds);
		asociacion = asociacionBO.recuperarAsociacion(String.valueOf(codigo.getIdAsociacion()));
		request.setAttribute(Ctes.ASOCIACION,asociacion);
		
		// Recuperar el idioma
		idiomaBO = new IdiomaBO(ds);
		idioma = idiomaBO.recuperarIdioma(idIdioma);
		request.setAttribute(Ctes.IDIOMA,idioma);
		
		return mapping.findForward(Ctes.EXITO);
}

}

