/*
 * AmbitoBO.java		03-ago-2005
 */
package codetic.bo;

import java.sql.Connection;
import java.util.*;

import javax.sql.DataSource;

import codetic.dao.*;
import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que contiene la lógica de negocio relacionada con la entidad Ambito. 
 *
 * @author Miguel Angel Serna Mata
 */
public class AmbitoBO {
	
	private DataSource dataSource;
	
	public AmbitoBO(DataSource dataSource) {
		this.dataSource = dataSource;
	}
	
	public void crearAmbito(AmbitoDTO ambito) throws Exception {
	    Connection con = null;
		AmbitoDAO ambitoDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			ambitoDAO = new AmbitoDAO(con);
			// Crear el ambito
			ambitoDAO.crear(ambito);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public AmbitoDTO recuperarAmbito(String idAmbito) throws Exception {
	    Connection con = null;
		AmbitoDAO ambitoDAO = null;
		AmbitoDTO ambito = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			ambitoDAO = new AmbitoDAO(con);
			// Recuperar el ambito
			ambito = ambitoDAO.recuperar(Integer.parseInt(idAmbito));
			JDBCUtils.commit(con);
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return ambito;
	}
	
	public void modificarAmbito(AmbitoDTO ambito) throws Exception {
	    Connection con = null;
		AmbitoDAO ambitoDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			ambitoDAO = new AmbitoDAO(con);
			// Modificar el ambito
			ambitoDAO.modificar(ambito);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public void eliminarAmbito(String idAmbito) throws Exception {
	    Connection con = null;
		AmbitoDAO ambitoDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			ambitoDAO = new AmbitoDAO(con);
			// Eliminar el ambito
			ambitoDAO.eliminar(Integer.parseInt(idAmbito));
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public Collection listaAmbitos() throws Exception {
		Connection con = null;
		AmbitoDAO ambitoDAO = null;
		Collection c = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			ambitoDAO = new AmbitoDAO(con);
			// Ejecuta la sentencia
			c = ambitoDAO.recuperarTodos();
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return c;
	}
	
	public Map mapAmbitos() throws Exception {
		Connection con = null;
		AmbitoDAO ambitoDAO = null;
		Map m = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			ambitoDAO = new AmbitoDAO(con);
			// Ejecuta la sentencia
			m = ambitoDAO.recuperarTodosMap();
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return m;
	}

}
