/*
 * AsociacionBO.java		03-ago-2005
 */
package codetic.bo;

import java.sql.Connection;
import java.util.*;

import javax.sql.DataSource;

import org.apache.commons.beanutils.BeanComparator;

import codetic.dao.*;
import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que contiene la lógica de negocio relacionada con la entidad Asociacion. 
 *
 * @author Miguel Angel Serna Mata
 */
public class AsociacionBO {
	
	private DataSource dataSource;
	
	public AsociacionBO(DataSource dataSource) {
		this.dataSource = dataSource;
	}
	
	public void crearAsociacion(AsociacionDTO Asociacion) throws Exception {
	    Connection con = null;
		AsociacionDAO AsociacionDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			AsociacionDAO = new AsociacionDAO(con);
			// Crear el Asociacion
			AsociacionDAO.crear(Asociacion);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public AsociacionDTO recuperarAsociacion(String idAsociacion) throws Exception {
	    Connection con = null;
		AsociacionDAO AsociacionDAO = null;
		AsociacionDTO Asociacion = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			AsociacionDAO = new AsociacionDAO(con);
			// Recuperar el Asociacion
			Asociacion = AsociacionDAO.recuperar(Integer.parseInt(idAsociacion));
			JDBCUtils.commit(con);
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return Asociacion;
	}
	
	public void modificarAsociacion(AsociacionDTO Asociacion) throws Exception {
	    Connection con = null;
		AsociacionDAO AsociacionDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			AsociacionDAO = new AsociacionDAO(con);
			// Modificar el Asociacion
			AsociacionDAO.modificar(Asociacion);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public void eliminarAsociacion(String idAsociacion) throws Exception {
	    Connection con = null;
		AsociacionDAO asociacionDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			asociacionDAO = new AsociacionDAO(con);
			// Eliminar el Asociacion
			asociacionDAO.eliminar(Integer.parseInt(idAsociacion));
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public Collection listaAsociaciones() throws Exception {
		Connection con = null;
		AsociacionDAO AsociacionDAO = null;
		Collection c = null;
		Comparator comparator = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			AsociacionDAO = new AsociacionDAO(con);
			// Ejecuta la sentencia
			c = AsociacionDAO.recuperarTodos();
			// Ordenar la lista
			comparator = new BeanComparator(Ctes.NOMBRE);
			Collections.sort((List)c,comparator);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return c;
	}

}
