/*
 * CodigoBO.java		03-ago-2005
 */
package codetic.bo;

import java.sql.Connection;
import java.util.*;

import javax.sql.DataSource;

import org.apache.commons.beanutils.BeanComparator;

import codetic.dao.*;
import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que contiene la lógica de negocio relacionada con la entidad Codigo. 
 *
 * @author Miguel Angel Serna Mata
 */
public class CodigoBO {
	
	private DataSource dataSource;
	
	public CodigoBO(DataSource dataSource) {
		this.dataSource = dataSource;
	}
	
	public void crearCodigo(CodigoDTO Codigo) throws Exception {
	    Connection con = null;
		CodigoDAO CodigoDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			CodigoDAO = new CodigoDAO(con);
			// Crear el Codigo
			CodigoDAO.crear(Codigo);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public CodigoDTO recuperarCodigo(String idCodigo) throws Exception {
	    Connection con = null;
		CodigoDAO CodigoDAO = null;
		CodigoDTO Codigo = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			CodigoDAO = new CodigoDAO(con);
			// Recuperar el Codigo
			Codigo = CodigoDAO.recuperar(Integer.parseInt(idCodigo));
			JDBCUtils.commit(con);
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return Codigo;
	}
	
	public void modificarCodigo(CodigoDTO Codigo) throws Exception {
	    Connection con = null;
		CodigoDAO CodigoDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			CodigoDAO = new CodigoDAO(con);
			// Modificar el Codigo
			CodigoDAO.modificar(Codigo);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public void eliminarCodigo(String idCodigo) throws Exception {
	    Connection con = null;
		CodigoDAO codigoDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			codigoDAO = new CodigoDAO(con);
			// Eliminar el Codigo
			codigoDAO.eliminar(Integer.parseInt(idCodigo));
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public Collection listaCodigosAsociacion(String idAsociacion) throws Exception {
		Connection con = null;
		CodigoDAO CodigoDAO = null;
		Collection c = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			CodigoDAO = new CodigoDAO(con);
			// Ejecuta la sentencia
			c = CodigoDAO.recuperarPorIdAsociacion(Integer.parseInt(idAsociacion));
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return c;
	}
	
	public Map mapCodigosYTraduccionesCodigos(String idAsociacion) throws Exception {
		Connection con = null;
		CodigoDAO codigoDAO = null;
		TraduccionCodigoDAO tradCodigoDAO = null;
		Collection codigos = null;
		Map map = null;
		Iterator iter = null;
		CodigoDTO codigo = null;
		Collection tradsCodigo = null;
		Comparator comparator = null;
		
		try {
		    // Obtener el DataSource
		    con = JDBCUtils.getConnection(dataSource);
			// Crear los DAOs
			codigoDAO = new CodigoDAO(con);
			tradCodigoDAO = new TraduccionCodigoDAO(con);
			// Recuperar los códigos de la asociación
			codigos = codigoDAO.recuperarPorIdAsociacion(Integer.parseInt(idAsociacion));
			// Crear un map ordenado
			comparator = new BeanComparator(Ctes.TITULO_ORIGINAL);
			map = new TreeMap(comparator);
			// Iterar sobre los códigos
			iter = codigos.iterator();
			while (iter.hasNext()) {
			    codigo = (CodigoDTO)iter.next();
			    // Recuperar las traducciones de ese código
			    tradsCodigo = new ArrayList();
			    tradsCodigo.addAll(tradCodigoDAO.recuperarPorIdCodigo(codigo.getIdCodigo()));
			    comparator = new BeanComparator(Ctes.TITULO);
				Collections.sort((List)tradsCodigo,comparator);
				// Añadir al map
				map.put(codigo,tradsCodigo);
			}
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return map;
	}

}
