/*
 * GrupoDeTemasBO.java		03-ago-2005
 */
package codetic.bo;

import java.sql.Connection;
import java.util.*;

import javax.sql.DataSource;

import org.apache.commons.beanutils.BeanComparator;
//import org.apache.commons.collections.comparators.ReverseComparator;

import codetic.dao.*;
import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que contiene la lógica de negocio relacionada con la entidad GrupoDeTemas. 
 *
 * @author Miguel Angel Serna Mata
 */
public class GrupoDeTemasBO {
	
	private DataSource dataSource;
	
	public GrupoDeTemasBO(DataSource dataSource) {
		this.dataSource = dataSource;
	}
	
	public void crearGrupoDeTemas(GrupoDeTemasDTO grupoDeTemas) throws Exception {
	    Connection con = null;
		GrupoDeTemasDAO grupoDeTemasDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			grupoDeTemasDAO = new GrupoDeTemasDAO(con);
			// Crear el grupoDeTemas
			grupoDeTemasDAO.crear(grupoDeTemas);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public GrupoDeTemasDTO recuperarGrupoDeTemas(String idGrupoDeTemas) throws Exception {
	    Connection con = null;
		GrupoDeTemasDAO grupoDeTemasDAO = null;
		GrupoDeTemasDTO grupoDeTemas = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			grupoDeTemasDAO = new GrupoDeTemasDAO(con);
			// Recuperar el grupoDeTemas
			grupoDeTemas = grupoDeTemasDAO.recuperar(Integer.parseInt(idGrupoDeTemas));
			JDBCUtils.commit(con);
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return grupoDeTemas;
	}
	
	public void modificarGrupoDeTemas(GrupoDeTemasDTO grupoDeTemas) throws Exception {
	    Connection con = null;
		GrupoDeTemasDAO grupoDeTemasDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			grupoDeTemasDAO = new GrupoDeTemasDAO(con);
			// Modificar el grupoDeTemas
			grupoDeTemasDAO.modificar(grupoDeTemas);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public void eliminarGrupoDeTemas(String idGrupoDeTemas) throws Exception {
	    Connection con = null;
		GrupoDeTemasDAO grupoDeTemasDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			grupoDeTemasDAO = new GrupoDeTemasDAO(con);
			// Eliminar el grupoDeTemas
			grupoDeTemasDAO.eliminar(Integer.parseInt(idGrupoDeTemas));
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public Collection listaGruposDeTemas() throws Exception {
		Connection con = null;
		GrupoDeTemasDAO grupoDeTemasDAO = null;
		Collection c = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			grupoDeTemasDAO = new GrupoDeTemasDAO(con);
			// Ejecuta la sentencia
			c = grupoDeTemasDAO.recuperarTodos();
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return c;
	}
	
	public Map mapGruposDeTemas() throws Exception {
		Connection con = null;
		GrupoDeTemasDAO grupoDeTemasDAO = null;
		Map m = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			grupoDeTemasDAO = new GrupoDeTemasDAO(con);
			// Ejecuta la sentencia
			m = grupoDeTemasDAO.recuperarTodosMap();
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return m;
	}
	
	public Map mapGruposYTemas() throws Exception {
		Connection con = null;
		GrupoDeTemasDAO grupoDeTemasDAO = null;
		TemaDAO temaDAO = null;
		Map map = null;
		Collection grupos = null;
		List temas = null;
		Iterator iter = null;
		GrupoDeTemasDTO grupo = null;
		Comparator comparator = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			// Crear el Map(Grupo,{Temas})
			grupoDeTemasDAO = new GrupoDeTemasDAO(con);
			temaDAO = new TemaDAO(con);
			grupos = grupoDeTemasDAO.recuperarTodos();
			comparator = new BeanComparator(Ctes.NOMBRE_ADMON);
			//comparator = new ReverseComparator(comparator);
			map = new TreeMap(comparator);
			iter = grupos.iterator();
			while (iter.hasNext()) {
			    grupo = (GrupoDeTemasDTO)iter.next();
			    temas = new ArrayList();
			    temas.addAll(temaDAO.recuperarPorIdGrupoDeTemas(grupo.getIdGrupoDeTemas()));
			    comparator = new BeanComparator(Ctes.NOMBRE_ADMON);
				Collections.sort((List)temas,comparator);
			    map.put(grupo,temas);
			}
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return map;
	}
	
	// Obtener parámetros para poner los temas en 2 columnas
	// Devuelve: [grupoi,temaj,ntemas]
	// -----------------------------------------------------
	public int[] dividirMapEn2(Map map) throws Exception {
	    int ngrupos, ntemas, parcial, total, grupoi, temaj = 0;
		boolean encontrado;
		int[] resp = new int[3];
		Collection grupos = null;
		Collection temas = null;
		GrupoDeTemasDTO grupo = null;
		Iterator iter = null;
		
		grupos = map.keySet();
		// 1. Obtener el total de items
		iter = grupos.iterator();
		ngrupos = grupos.size();
		ntemas = 0;
		while (iter.hasNext()) {
		    temas = (Collection)map.get(iter.next());
		    ntemas += temas.size();
		}
		total = ngrupos + ntemas;
		// 2. Obtener el grupoi y el temaj
		iter = grupos.iterator();
		grupoi = 0;
		encontrado = false;
		parcial = 0;
		while (iter.hasNext() && !encontrado) {
		    temas = (Collection)map.get(iter.next());
		    parcial = parcial + 1 + temas.size();
		    if (parcial >= (total/2)) {
		        temaj = temas.size() - (parcial - total/2) - 1;
		        if (temaj == 0) {
		            //grupoi--;
		            temaj = temas.size();
		        }
		        encontrado = true;
		    } else {
		        grupoi++;
		    }
		}
		
		resp[0] = grupoi;
		resp[1] = temaj;
		resp[2] = ntemas;
		return resp;
	}
}
