/*
 * IdiomaBO.java		03-ago-2005
 */
package codetic.bo;

import java.sql.Connection;
import java.util.*;

import javax.sql.DataSource;
//import org.apache.commons.beanutils.BeanComparator;

import codetic.dao.*;
import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que contiene la l�gica de negocio relacionada con la entidad Idioma. 
 *
 * @author Miguel Angel Serna Mata
 */
public class IdiomaBO {
	
	private DataSource dataSource;
	
	public IdiomaBO(DataSource dataSource) {
		this.dataSource = dataSource;
	}
	
	public void crearIdioma(IdiomaDTO idioma) throws Exception {
	    Connection con = null;
		IdiomaDAO idiomaDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			idiomaDAO = new IdiomaDAO(con);
			// Crear el idioma
			idiomaDAO.crear(idioma);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public IdiomaDTO recuperarIdioma(String idIdioma) throws Exception {
	    Connection con = null;
		IdiomaDTO idioma = null;
		IdiomaDAO idiomaDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			idiomaDAO = new IdiomaDAO(con);
			// Recuperar el idioma
			idioma = idiomaDAO.recuperar(idIdioma);
			JDBCUtils.commit(con);
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return idioma;
	}
	
	public void modificarIdioma(IdiomaDTO idioma) throws Exception {
	    Connection con = null;
		IdiomaDAO idiomaDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			idiomaDAO = new IdiomaDAO(con);
			// Modificar el idioma
			idiomaDAO.modificar(idioma);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public void eliminarIdioma(String idIdioma) throws Exception {
	    Connection con = null;
		IdiomaDAO idiomaDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			idiomaDAO = new IdiomaDAO(con);
			// Eliminar el idioma
			idiomaDAO.eliminar(idIdioma);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public Collection listaIdiomas() throws Exception {
		Connection con = null;
		IdiomaDAO idiomaDAO = null;
		Collection c = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			idiomaDAO = new IdiomaDAO(con);
			// Ejecuta la sentencia
			c = idiomaDAO.recuperarTodos();
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return c;
	}

}
