/*
 * SeccionBO.java		03-ago-2005
 */
package codetic.bo;

import java.sql.Connection;
import java.util.*;

import javax.sql.DataSource;

import org.apache.commons.beanutils.BeanComparator;
import org.apache.commons.collections.comparators.*;

import codetic.dao.*;
import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que contiene la lógica de negocio relacionada con la entidad Seccion. 
 *
 * @author Miguel Angel Serna Mata
 */
public class SeccionBO {
	
	private DataSource dataSource;
	
	public SeccionBO(DataSource dataSource) {
		this.dataSource = dataSource;
	}
	
	public void crearSeccion(SeccionDTO seccion) throws Exception {
	    Connection con = null;
		SeccionDAO SeccionDAO = null;
		SeccionTemaDAO seccionTemaDAO = null;
		SeccionTemaDTO seccionTema = null;
		String[] temasSeleccionados = null;
		
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			SeccionDAO = new SeccionDAO(con);
			// Crear la Seccion
			SeccionDAO.crear(seccion);
			// Crear las asociaciones Seccion-Tema
			temasSeleccionados = seccion.getTemas();
			seccionTemaDAO = new SeccionTemaDAO(con);
			for (int i=0; i<temasSeleccionados.length; i++) {
			    seccionTema = new SeccionTemaDTO();
			    seccionTema.setNivel(seccion.getNivel());
			    seccionTema.setIdCodigo(seccion.getIdCodigo());
			    seccionTema.setIdTema(Integer.parseInt((String) temasSeleccionados[i]));
			    seccionTemaDAO.crear(seccionTema);
			}
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public SeccionDTO recuperarSeccion(String nivel, String idCodigo) throws Exception {
	    Connection con = null;
		SeccionDAO seccionDAO = null;
		SeccionDTO seccion = null;
		SeccionTemaDAO seccionTemaDAO = null;
		SeccionTemaDTO seccionTema = null;
		Collection temasSeccion = null;
		Iterator iter = null;
		String[] temasSeleccionados = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			seccionDAO = new SeccionDAO(con);
			// Recuperar el Seccion
			seccion = seccionDAO.recuperar(nivel,Integer.parseInt(idCodigo));
			// Recuperar temas asociados a la sección
			seccionTemaDAO = new SeccionTemaDAO(con);
			temasSeccion = seccionTemaDAO.recuperarPorNivelYIdCodigo(nivel,Integer.parseInt(idCodigo));
			temasSeleccionados = new String[temasSeccion.size()];
			iter = temasSeccion.iterator();
			for (int i= 0; i<temasSeccion.size(); i++) {
			    seccionTema = (SeccionTemaDTO)iter.next();
			    temasSeleccionados[i] = String.valueOf(seccionTema.getIdTema());
			}
			seccion.setTemas(temasSeleccionados);
			JDBCUtils.commit(con);
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return seccion;
	}
	
	public void modificarSeccion(SeccionDTO seccion) throws Exception {
	    Connection con = null;
		SeccionDAO SeccionDAO = null;
		SeccionTemaDAO seccionTemaDAO = null;
		SeccionTemaDTO seccionTema = null;
		Collection temasSeccion = null;
		Iterator iter = null;
		String[] temasSeleccionados = null;
		TemaDAO temaDAO = null;
		TemaDTO tema = null;
		Collection temas = null;
		String nivel = null;
		int idCodigo, idTema;
		boolean temaSeleccionado;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			SeccionDAO = new SeccionDAO(con);
			// Modificar el Seccion
			SeccionDAO.modificar(seccion);
			// ----------------------------------------
			// Actualizar las asociaciones Seccion-Tema
			// ----------------------------------------
			// 1. Recuperar los temas asociados seleccionados
			temasSeleccionados = seccion.getTemas();
			// 2. Crear variables nivel y idCodigo
			nivel = seccion.getNivel();
			idCodigo = seccion.getIdCodigo();
			// 3. Recuperar los temas
			temaDAO = new TemaDAO(con);
			temas = temaDAO.recuperarTodos();
			// 4. Recorrer la colección temas
			seccionTemaDAO = new SeccionTemaDAO(con);
			iter = temas.iterator();
			while (iter.hasNext()) {
			    // 4.1. Recuperar el tema
			    tema = (TemaDTO)iter.next();
			    // 4.2. Establecer el id del tema
			    idTema = tema.getIdTema();
			    // 4.3. Recuperar la asociación Seccion-Tema
			    seccionTema = seccionTemaDAO.recuperar(nivel,idCodigo,idTema);
			    // 4.4. Ver si el tema sobre el que iteramos está dentro de temasSeleccionados
			    //temaSeleccionado = Arrays.binarySearch(temasSeleccionados,String.valueOf(idTema)) != -1;
			    int i = 0;
			    temaSeleccionado = false;
			    while(!temaSeleccionado && (i<temasSeleccionados.length)) {
			        temaSeleccionado = (Integer.parseInt(temasSeleccionados[i]) == idTema);
			        i++;
			    }
			    // 4.5. Crear o eliminar la asociación
			    if (temaSeleccionado && (seccionTema == null)) {
			        // 4.5.a. Crear asociación Seccion-Tema
			        seccionTema = new SeccionTemaDTO();
			        seccionTema.setNivel(nivel);
			        seccionTema.setIdCodigo(idCodigo);
			        seccionTema.setIdTema(idTema);
			        seccionTemaDAO.crear(seccionTema);
			    } else if (!temaSeleccionado && (seccionTema != null)) {
			    	// 4.5.b. Eliminar la asociación Seccion-Tema
			    	seccionTemaDAO.eliminar(nivel,idCodigo,idTema);
			    }
			}
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public void modificarSeccion(SeccionDTO seccion, String nivelAntiguo) throws Exception {
	    Connection con = null;
		SeccionDAO SeccionDAO = null;
		SeccionTemaDAO seccionTemaDAO = null;
		SeccionTemaDTO seccionTema = null;
		Collection temasSeccion = null;
		Iterator iter = null;
		String[] temasSeleccionados = null;
		TemaDAO temaDAO = null;
		TemaDTO tema = null;
		Collection temas = null;
		String nivel = null;
		int idCodigo, idTema;
		boolean temaSeleccionado;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			SeccionDAO = new SeccionDAO(con);
			// Modificar el Seccion
			SeccionDAO.modificar(seccion,nivelAntiguo);
			// ----------------------------------------
			// Actualizar las asociaciones Seccion-Tema
			// ----------------------------------------
			// 1. Recuperar los temas asociados seleccionados
			temasSeleccionados = seccion.getTemas();
			// 2. Crear variables nivel y idCodigo
			nivel = seccion.getNivel();
			idCodigo = seccion.getIdCodigo();
			// 3. Recuperar los temas
			temaDAO = new TemaDAO(con);
			temas = temaDAO.recuperarTodos();
			// 4. Recorrer la colección temas
			seccionTemaDAO = new SeccionTemaDAO(con);
			iter = temas.iterator();
			while (iter.hasNext()) {
			    // 4.1. Recuperar el tema
			    tema = (TemaDTO)iter.next();
			    // 4.2. Establecer el id del tema
			    idTema = tema.getIdTema();
			    // 4.3. Recuperar la asociación Seccion-Tema
			    seccionTema = seccionTemaDAO.recuperar(nivel,idCodigo,idTema);
			    // 4.4. Ver si el tema sobre el que iteramos está dentro de temasSeleccionados
			    //temaSeleccionado = Arrays.binarySearch(temasSeleccionados,String.valueOf(idTema)) != -1;
			    int i = 0;
			    temaSeleccionado = false;
			    while(!temaSeleccionado && (i<temasSeleccionados.length)) {
			        temaSeleccionado = (Integer.parseInt(temasSeleccionados[i]) == idTema);
			        i++;
			    }
			    // 4.5. Crear o eliminar la asociación
			    if (temaSeleccionado && (seccionTema == null)) {
			        // 4.5.a. Crear asociación Seccion-Tema
			        seccionTema = new SeccionTemaDTO();
			        seccionTema.setNivel(nivel);
			        seccionTema.setIdCodigo(idCodigo);
			        seccionTema.setIdTema(idTema);
			        seccionTemaDAO.crear(seccionTema);
			    } else if (!temaSeleccionado && (seccionTema != null)) {
			    	// 4.5.b. Eliminar la asociación Seccion-Tema
			    	seccionTemaDAO.eliminar(nivel,idCodigo,idTema);
			    }
			}
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public void eliminarSeccion(String nivel, String idCodigo) throws Exception {
	    Connection con = null;
		SeccionDAO seccionDAO = null;
		SeccionTemaDAO seccionTemaDAO = null;
		SeccionTemaDTO seccion = null;
		SeccionTemaDTO seccionTema = null;
		String[] temasSeleccionados = null;
		int tema;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			// Eliminar las asociaciones Seccion-Tema
			seccionTemaDAO = new SeccionTemaDAO(con);
			seccionTemaDAO.eliminarPorNivelYIdCodigo(nivel,Integer.parseInt(idCodigo));
			// Eliminar la Seccion
			seccionDAO = new SeccionDAO(con);
			seccionDAO.eliminar(nivel,Integer.parseInt(idCodigo));
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public Collection listaSeccionesCodigo(String idCodigo) throws Exception {
		Connection con = null;
		SeccionDAO SeccionDAO = null;
		Collection c = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			SeccionDAO = new SeccionDAO(con);
			// Ejecuta la sentencia
			c = SeccionDAO.recuperarPorIdCodigo(Integer.parseInt(idCodigo));
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return c;
	}

	public Map mapSeccionesYTraducciones(String idIdioma, String idCodigo) throws Exception {
		Connection con = null;
		SeccionDAO seccionDAO = null;
		TraduccionSeccionDAO tradSeccionDAO = null;
		Collection secciones = null;
		Map map = null;
		Iterator iter = null;
		SeccionDTO seccion = null;
		TraduccionSeccionDTO tradSeccion = null;
		Comparator comparator = null;
		
		try {
		    // Obtener el DataSource
		    con = JDBCUtils.getConnection(dataSource);
			// Crear los DAOs
			seccionDAO = new SeccionDAO(con);
			tradSeccionDAO = new TraduccionSeccionDAO(con);
			// Recuperar las secciones
			secciones = seccionDAO.recuperarPorIdCodigo(Integer.parseInt(idCodigo));
			// Crear un map ordenado
			comparator = new BeanComparator("nivel");
			map = new TreeMap(comparator);
			// Iterar sobre las secciones
			iter = secciones.iterator();
			while (iter.hasNext()) {
			    seccion = (SeccionDTO)iter.next();
			    // Recuperar la traducción asociada
			    tradSeccion = tradSeccionDAO.recuperar(idIdioma,Integer.parseInt(idCodigo),seccion.getNivel());
			    map.put(seccion,tradSeccion);
			}
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return map;
	}
	
	// El elemento de más arriba de la pila es la primera sección
	public Stack pilaSeccionesSinTraducir(String idIdioma, String idCodigo) throws Exception {
	    Connection con = null;
		SeccionDAO seccionDAO = null;
		TraduccionSeccionDAO tradSeccionDAO = null;
		Collection secciones = null;
		Stack pila = null;
		Iterator iter = null;
		SeccionDTO seccion = null;
		TraduccionSeccionDTO tradSeccion = null;
		Comparator comparator = null;
		SeccionTemaDAO seccionTemaDAO = null;
		SeccionTemaDTO seccionTema = null;
		Collection temasSeccion = null;
		String[] temasSeleccionados = null;
		
		try {
		    // Obtener el DataSource
		    con = JDBCUtils.getConnection(dataSource);
			// Crear los DAOs
			seccionDAO = new SeccionDAO(con);
			tradSeccionDAO = new TraduccionSeccionDAO(con);
			seccionTemaDAO = new SeccionTemaDAO(con);
			// Recuperar las secciones
			secciones = seccionDAO.recuperarPorIdCodigo(Integer.parseInt(idCodigo));
			// Ordenar las secciones
			comparator = new BeanComparator("nivel");
			comparator = new ReverseComparator(comparator);
			Collections.sort((List)secciones,comparator);
			// Crear la pila vacía
			pila = new Stack();
			// Iterar sobre las secciones
			iter = secciones.iterator();
			while (iter.hasNext()) {
			    seccion = (SeccionDTO)iter.next();
			    // Recuperar la traducción asociada
			    tradSeccion = tradSeccionDAO.recuperar(idIdioma,Integer.parseInt(idCodigo),seccion.getNivel());
			    // Añadir las secciones sin traducción
			    if (tradSeccion == null) {
			        // Recuperar temas asociados a la sección
					temasSeccion = seccionTemaDAO.recuperarPorNivelYIdCodigo(seccion.getNivel(),Integer.parseInt(idCodigo));
					temasSeleccionados = new String[temasSeccion.size()];
					Iterator iter2 = temasSeccion.iterator();
					for (int i= 0; i<temasSeccion.size(); i++) {
					    seccionTema = (SeccionTemaDTO)iter2.next();
					    temasSeleccionados[i] = String.valueOf(seccionTema.getIdTema());
					}
					seccion.setTemas(temasSeleccionados);
					// Añadir la sección
			        pila.push(seccion);
			    }
			}
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return pila;
	}
}
