/*
 * SeccionTemaBO.java		03-ago-2005
 */
package codetic.bo;

import java.sql.Connection;
import javax.sql.DataSource;
import java.util.*;

import codetic.dao.*;
import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que contiene la lógica de negocio relacionada con la entidad SeccionTema. 
 *
 * @author Miguel Angel Serna Mata
 */
public class SeccionTemaBO {
	
	private DataSource dataSource;
	
	public SeccionTemaBO(DataSource dataSource) {
		this.dataSource = dataSource;
	}
	
	public void crearSeccionTema(SeccionTemaDTO SeccionTema) throws Exception {
	    Connection con = null;
		SeccionTemaDAO seccionTemaDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			seccionTemaDAO = new SeccionTemaDAO(con);
			// Crear el SeccionTema
			seccionTemaDAO.crear(SeccionTema);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public SeccionTemaDTO recuperarSeccionTema(String nivel, String idCodigo, String idTema) throws Exception {
	    Connection con = null;
		SeccionTemaDAO seccionTemaDAO = null;
		SeccionTemaDTO seccionTema = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			seccionTemaDAO = new SeccionTemaDAO(con);
			// Recuperar el SeccionTema
			seccionTema = seccionTemaDAO.recuperar(nivel,Integer.parseInt(idCodigo),Integer.parseInt(idTema));
			JDBCUtils.commit(con);
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return seccionTema;
	}
	
	public void eliminarSeccionTema(String nivel, String idCodigo, String idTema) throws Exception {
	    Connection con = null;
		SeccionTemaDAO seccionDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			seccionDAO = new SeccionTemaDAO(con);
			// Eliminar el SeccionTema
			seccionDAO.eliminar(nivel,Integer.parseInt(idCodigo),Integer.parseInt(idTema));
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public Collection listaTemasSeccion(String nivel, String idCodigo) throws Exception {
		Connection con = null;
		SeccionTemaDAO seccionTemaDAO = null;
		Collection c = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			seccionTemaDAO = new SeccionTemaDAO(con);
			// Ejecuta la sentencia
			c = seccionTemaDAO.recuperarPorNivelYIdCodigo(nivel,Integer.parseInt(idCodigo));
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return c;
	}

}
