/*
 * TemaBO.java		03-ago-2005
 */
package codetic.bo;

import java.sql.Connection;
import java.util.*;

import javax.sql.DataSource;

import codetic.dao.*;
import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que contiene la lógica de negocio relacionada con la entidad Tema. 
 *
 * @author Miguel Angel Serna Mata
 */
public class TemaBO {
	
	private DataSource dataSource;
	
	public TemaBO(DataSource dataSource) {
		this.dataSource = dataSource;
	}
	
	public void crearTema(TemaDTO Tema) throws Exception {
	    Connection con = null;
		TemaDAO TemaDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			TemaDAO = new TemaDAO(con);
			// Crear el Tema
			TemaDAO.crear(Tema);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public TemaDTO recuperarTema(String idTema) throws Exception {
	    Connection con = null;
		TemaDAO TemaDAO = null;
		TemaDTO Tema = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			TemaDAO = new TemaDAO(con);
			// Recuperar el Tema
			Tema = TemaDAO.recuperar(Integer.parseInt(idTema));
			JDBCUtils.commit(con);
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return Tema;
	}
	
	public void modificarTema(TemaDTO Tema) throws Exception {
	    Connection con = null;
		TemaDAO TemaDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			TemaDAO = new TemaDAO(con);
			// Modificar el Tema
			TemaDAO.modificar(Tema);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public void eliminarTema(String idTema) throws Exception {
	    Connection con = null;
		TemaDAO TemaDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			TemaDAO = new TemaDAO(con);
			// Eliminar el Tema
			TemaDAO.eliminar(Integer.parseInt(idTema));
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public Collection listaTemas() throws Exception {
		Connection con = null;
		TemaDAO TemaDAO = null;
		Collection c = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			TemaDAO = new TemaDAO(con);
			// Ejecuta la sentencia
			c = TemaDAO.recuperarTodos();
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return c;
	}
	
	public Collection listaTemasGrupo(String idGrupoDeTemas) throws Exception {
		Connection con = null;
		TemaDAO temaDAO = null;
		Collection c = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			temaDAO = new TemaDAO(con);
			// Ejecuta la sentencia
			c = temaDAO.recuperarPorIdGrupoDeTemas(Integer.parseInt(idGrupoDeTemas));
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return c;
	}

}
