/*
 * TraduccionAmbitoBO.java		03-ago-2005
 */
package codetic.bo;

import java.sql.Connection;
import java.util.*;

import javax.sql.DataSource;

import org.apache.commons.beanutils.BeanComparator;

import codetic.dao.*;
import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que contiene la lógica de negocio relacionada con la entidad TraduccionAmbito. 
 *
 * @author Miguel Angel Serna Mata
 */
public class TraduccionAmbitoBO {
	
	private DataSource dataSource;
	
	public TraduccionAmbitoBO(DataSource dataSource) {
		this.dataSource = dataSource;
	}
	
	public void crearTraduccionAmbito(TraduccionAmbitoDTO tradAmbito) throws Exception {
	    Connection con = null;
		TraduccionAmbitoDAO tradAmbitoDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradAmbitoDAO = new TraduccionAmbitoDAO(con);
			// Crear la traducción
			tradAmbitoDAO.crear(tradAmbito);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public TraduccionAmbitoDTO recuperarTraduccionAmbito(String idIdioma, String idAmbito) throws Exception {
	    Connection con = null;
		TraduccionAmbitoDAO tradAmbitoDAO = null;
		TraduccionAmbitoDTO tradAmbito = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradAmbitoDAO = new TraduccionAmbitoDAO(con);
			// Recuperar la traducción
			tradAmbito = tradAmbitoDAO.recuperar(idIdioma,Integer.parseInt(idAmbito));
			JDBCUtils.commit(con);
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return tradAmbito;
	}
	
	public void modificarTraduccionAmbito(TraduccionAmbitoDTO tradAmbito) throws Exception {
	    Connection con = null;
		TraduccionAmbitoDAO tradAmbitoDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradAmbitoDAO = new TraduccionAmbitoDAO(con);
			// Modificar la traducción
			tradAmbitoDAO.modificar(tradAmbito);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public void eliminarTraduccionAmbito(String idIdioma, String idAmbito) throws Exception {
	    Connection con = null;
		TraduccionAmbitoDAO tradAmbitoDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradAmbitoDAO = new TraduccionAmbitoDAO(con);
			// Eliminar el tradAmbito
			tradAmbitoDAO.eliminar(idIdioma,Integer.parseInt(idAmbito));
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public Collection listaTraduccionesAmbito(String idAmbito) throws Exception {
		Connection con = null;
		TraduccionAmbitoDAO tradAmbitoDAO = null;
		Collection c = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradAmbitoDAO = new TraduccionAmbitoDAO(con);
			// Ejecuta la sentencia
			c = tradAmbitoDAO.recuperarPorIdAmbito(Integer.parseInt(idAmbito));
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return c;
	}

	// Sólo añade los ámbitos que tienen asociaciones
	public Map mapTraduccionesAmbitosYAsociaciones(String idIdioma) throws Exception {
		Connection con = null;
		TraduccionAmbitoDAO tradAmbitoDAO = null;
		AsociacionDAO asociacionDAO = null;
		Collection tradAmbitos = null;
		Map map = null;
		Iterator iter = null;
		TraduccionAmbitoDTO tradAmbito = null;
		Collection asociaciones = null;
		Comparator comparator = null;
		
		try {
		    // Obtener el DataSource
		    con = JDBCUtils.getConnection(dataSource);
			// Crear los DAOs
			tradAmbitoDAO = new TraduccionAmbitoDAO(con);
			asociacionDAO = new AsociacionDAO(con);
			// Recuperar las traducciones de ámbitos
			tradAmbitos = tradAmbitoDAO.recuperarPorIdIdioma(idIdioma);
			// Crear un map ordenado
			comparator = new BeanComparator(Ctes.NOMBRE);
			map = new TreeMap(comparator);
			// Iterar sobre las secciones
			iter = tradAmbitos.iterator();
			while (iter.hasNext()) {
			    tradAmbito = (TraduccionAmbitoDTO)iter.next();
			    // Recuperar las asociaciones de ese ámbito 
			    asociaciones = new ArrayList();
			    asociaciones.addAll(asociacionDAO.recuperarPorIdAmbito(tradAmbito.getIdAmbito()));
			    comparator = new BeanComparator(Ctes.NOMBRE);
				Collections.sort((List)asociaciones,comparator);
				// Añadir al map
				if (!asociaciones.isEmpty())
				    map.put(tradAmbito,asociaciones);
			}
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return map;
	}
}
