/*
 * TraduccionAsociacionBO.java		03-ago-2005
 */
package codetic.bo;

import java.sql.Connection;
import java.util.*;

import javax.sql.DataSource;

import codetic.dao.*;
import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que contiene la lógica de negocio relacionada con la entidad TraduccionAsociacion. 
 *
 * @author Miguel Angel Serna Mata
 */
public class TraduccionAsociacionBO {
	
	private DataSource dataSource;
	
	public TraduccionAsociacionBO(DataSource dataSource) {
		this.dataSource = dataSource;
	}
	
	public void crearTraduccionAsociacion(TraduccionAsociacionDTO tradAsociacion) throws Exception {
	    Connection con = null;
		TraduccionAsociacionDAO tradAsociacionDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradAsociacionDAO = new TraduccionAsociacionDAO(con);
			// Crear la traducción
			tradAsociacionDAO.crear(tradAsociacion);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public TraduccionAsociacionDTO recuperarTraduccionAsociacion(String idIdioma, String idAsociacion) throws Exception {
	    Connection con = null;
		TraduccionAsociacionDAO tradAsociacionDAO = null;
		TraduccionAsociacionDTO tradAsociacion = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradAsociacionDAO = new TraduccionAsociacionDAO(con);
			// Recuperar la traducción
			tradAsociacion = tradAsociacionDAO.recuperar(idIdioma,Integer.parseInt(idAsociacion));
			JDBCUtils.commit(con);
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return tradAsociacion;
	}
	
	public void modificarTraduccionAsociacion(TraduccionAsociacionDTO tradAsociacion) throws Exception {
	    Connection con = null;
		TraduccionAsociacionDAO tradAsociacionDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradAsociacionDAO = new TraduccionAsociacionDAO(con);
			// Modificar la traducción
			tradAsociacionDAO.modificar(tradAsociacion);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public void eliminarTraduccionAsociacion(String idIdioma, String idAsociacion) throws Exception {
	    Connection con = null;
		TraduccionAsociacionDAO tradAsociacionDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradAsociacionDAO = new TraduccionAsociacionDAO(con);
			// Eliminar el tradAsociacion
			tradAsociacionDAO.eliminar(idIdioma,Integer.parseInt(idAsociacion));
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public Collection listaTraduccionesAsociacion(String idAsociacion) throws Exception {
		Connection con = null;
		TraduccionAsociacionDAO tradAsociacionDAO = null;
		Collection c = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradAsociacionDAO = new TraduccionAsociacionDAO(con);
			// Ejecuta la sentencia
			c = tradAsociacionDAO.recuperarPorIdAsociacion(Integer.parseInt(idAsociacion));
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return c;
	}

}
