/*
 * TraduccionCodigoBO.java		03-ago-2005
 */
package codetic.bo;

import java.sql.Connection;
import java.util.*;

import javax.sql.DataSource;

import codetic.dao.*;
import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que contiene la lógica de negocio relacionada con la entidad TraduccionCodigo. 
 *
 * @author Miguel Angel Serna Mata
 */
public class TraduccionCodigoBO {
	
	private DataSource dataSource;
	
	public TraduccionCodigoBO(DataSource dataSource) {
		this.dataSource = dataSource;
	}
	
	public void crearTraduccionCodigo(TraduccionCodigoDTO tradCodigo) throws Exception {
	    Connection con = null;
		TraduccionCodigoDAO tradCodigoDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradCodigoDAO = new TraduccionCodigoDAO(con);
			// Crear la traducción
			tradCodigoDAO.crear(tradCodigo);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public TraduccionCodigoDTO recuperarTraduccionCodigo(String idIdioma, String idCodigo) throws Exception {
	    Connection con = null;
		TraduccionCodigoDAO tradCodigoDAO = null;
		TraduccionCodigoDTO tradCodigo = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradCodigoDAO = new TraduccionCodigoDAO(con);
			// Recuperar la traducción
			tradCodigo = tradCodigoDAO.recuperar(idIdioma,Integer.parseInt(idCodigo));
			JDBCUtils.commit(con);
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return tradCodigo;
	}
	
	public void modificarTraduccionCodigo(TraduccionCodigoDTO tradCodigo) throws Exception {
	    Connection con = null;
		TraduccionCodigoDAO tradCodigoDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradCodigoDAO = new TraduccionCodigoDAO(con);
			// Modificar la traducción
			tradCodigoDAO.modificar(tradCodigo);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public void eliminarTraduccionCodigo(String idIdioma, String idCodigo) throws Exception {
	    Connection con = null;
		TraduccionCodigoDAO tradCodigoDAO = null;
		TraduccionSeccionDAO tradSeccionDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradCodigoDAO = new TraduccionCodigoDAO(con);
			// Eliminar el tradCodigo
			tradCodigoDAO.eliminar(idIdioma,Integer.parseInt(idCodigo));
			// Eliminar las traducciones de secciones asociadas
			tradSeccionDAO = new TraduccionSeccionDAO(con);
			tradSeccionDAO.eliminarPorIdIdiomaYIdCodigo(idIdioma,Integer.parseInt(idCodigo));
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public Collection listaTraduccionesCodigo(String idCodigo) throws Exception {
		Connection con = null;
		TraduccionCodigoDAO tradCodigoDAO = null;
		Collection c = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradCodigoDAO = new TraduccionCodigoDAO(con);
			// Ejecuta la sentencia
			c = tradCodigoDAO.recuperarPorIdCodigo(Integer.parseInt(idCodigo));
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return c;
	}

}
