/*
 * TraduccionGrupoDeTemasBO.java		03-ago-2005
 */
package codetic.bo;

import java.sql.Connection;
import java.util.*;

import javax.sql.DataSource;

import org.apache.commons.beanutils.BeanComparator;

import codetic.dao.*;
import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que contiene la lógica de negocio relacionada con la entidad TraduccionGrupoDeTemas. 
 *
 * @author Miguel Angel Serna Mata
 */
public class TraduccionGrupoDeTemasBO {
	
	private DataSource dataSource;
	
	public TraduccionGrupoDeTemasBO(DataSource dataSource) {
		this.dataSource = dataSource;
	}
	
	public void crearTraduccionGrupoDeTemas(TraduccionGrupoDeTemasDTO tradGrupoDeTemas) throws Exception {
	    Connection con = null;
		TraduccionGrupoDeTemasDAO tradGrupoDeTemasDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradGrupoDeTemasDAO = new TraduccionGrupoDeTemasDAO(con);
			// Crear la traducción
			tradGrupoDeTemasDAO.crear(tradGrupoDeTemas);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public TraduccionGrupoDeTemasDTO recuperarTraduccionGrupoDeTemas(String idIdioma, String idGrupoDeTemas) throws Exception {
	    Connection con = null;
		TraduccionGrupoDeTemasDAO tradGrupoDeTemasDAO = null;
		TraduccionGrupoDeTemasDTO tradGrupoDeTemas = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradGrupoDeTemasDAO = new TraduccionGrupoDeTemasDAO(con);
			// Recuperar la traducción
			tradGrupoDeTemas = tradGrupoDeTemasDAO.recuperar(idIdioma,Integer.parseInt(idGrupoDeTemas));
			JDBCUtils.commit(con);
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return tradGrupoDeTemas;
	}
	
	public void modificarTraduccionGrupoDeTemas(TraduccionGrupoDeTemasDTO tradGrupoDeTemas) throws Exception {
	    Connection con = null;
		TraduccionGrupoDeTemasDAO tradGrupoDeTemasDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradGrupoDeTemasDAO = new TraduccionGrupoDeTemasDAO(con);
			// Modificar la traducción
			tradGrupoDeTemasDAO.modificar(tradGrupoDeTemas);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public void eliminarTraduccionGrupoDeTemas(String idIdioma, String idGrupoDeTemas) throws Exception {
	    Connection con = null;
		TraduccionGrupoDeTemasDAO tradGrupoDeTemasDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradGrupoDeTemasDAO = new TraduccionGrupoDeTemasDAO(con);
			// Eliminar el tradGrupoDeTemas
			tradGrupoDeTemasDAO.eliminar(idIdioma,Integer.parseInt(idGrupoDeTemas));
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public Collection listaTraduccionesGrupoDeTemas(String idGrupoDeTemas) throws Exception {
		Connection con = null;
		TraduccionGrupoDeTemasDAO tradGrupoDeTemasDAO = null;
		Collection c = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradGrupoDeTemasDAO = new TraduccionGrupoDeTemasDAO(con);
			// Ejecuta la sentencia
			c = tradGrupoDeTemasDAO.recuperarPorIdGrupoDeTemas(Integer.parseInt(idGrupoDeTemas));
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return c;
	}
	
	// Sólo añade los grupos que tienen temas
	public Map mapTraduccionesGruposYTraduccionesTemas(String idIdioma) throws Exception {
		Connection con = null;
		TraduccionGrupoDeTemasDAO tradGrupoDeTemasDAO = null;
		TraduccionTemaDAO tradTemaDAO = null;
		Collection tradsGrupoDeTemas = null;
		Map map = null;
		Iterator iter = null;
		TraduccionGrupoDeTemasDTO tradGrupoDeTemas = null;
		Collection tradsTemas = null;
		Comparator comparator = null;
		
		try {
		    // Obtener el DataSource
		    con = JDBCUtils.getConnection(dataSource);
			// Crear los DAOs
			tradGrupoDeTemasDAO = new TraduccionGrupoDeTemasDAO(con);
			tradTemaDAO = new TraduccionTemaDAO(con);
			// Recuperar las traducciones de grupos
			tradsGrupoDeTemas = tradGrupoDeTemasDAO.recuperarPorIdIdioma(idIdioma);
			// Crear un map ordenado
			comparator = new BeanComparator(Ctes.NOMBRE);
			map = new TreeMap(comparator);
			// Iterar sobre los grupos
			iter = tradsGrupoDeTemas.iterator();
			while (iter.hasNext()) {
			    tradGrupoDeTemas = (TraduccionGrupoDeTemasDTO)iter.next();
			    // Recuperar los temas de ese grupo
			    tradsTemas = new ArrayList();
			    tradsTemas.addAll(tradTemaDAO.recuperarPorIdGrupoDeTemasYIdIdioma(tradGrupoDeTemas.getIdGrupoDeTemas(),idIdioma));
			    comparator = new BeanComparator(Ctes.NOMBRE);
				Collections.sort((List)tradsTemas,comparator);
				// Añadir al map
				if (!tradsTemas.isEmpty())
				    map.put(tradGrupoDeTemas,tradsTemas);
			}
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return map;
	}

}
