/*
 * TraduccionSeccionBO.java		03-ago-2005
 */
package codetic.bo;

import java.sql.Connection;
import java.util.*;

import javax.sql.DataSource;

import codetic.dao.*;
import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que contiene la lógica de negocio relacionada con la entidad TraduccionSeccion. 
 *
 * @author Miguel Angel Serna Mata
 */
public class TraduccionSeccionBO {
	
	private DataSource dataSource;
	
	public TraduccionSeccionBO(DataSource dataSource) {
		this.dataSource = dataSource;
	}
	
	public void crearTraduccionSeccion(TraduccionSeccionDTO tradSeccion) throws Exception {
	    Connection con = null;
		TraduccionSeccionDAO tradSeccionDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradSeccionDAO = new TraduccionSeccionDAO(con);
			// Crear la traducción
			tradSeccionDAO.crear(tradSeccion);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public TraduccionSeccionDTO recuperarTraduccionSeccion(String idIdioma, String idCodigo, String nivel) throws Exception {
	    Connection con = null;
		TraduccionSeccionDAO tradSeccionDAO = null;
		TraduccionSeccionDTO tradSeccion = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradSeccionDAO = new TraduccionSeccionDAO(con);
			// Recuperar la traducción
			tradSeccion = tradSeccionDAO.recuperar(idIdioma,Integer.parseInt(idCodigo),nivel);
			JDBCUtils.commit(con);
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return tradSeccion;
	}
	
	public void modificarTraduccionSeccion(TraduccionSeccionDTO tradSeccion) throws Exception {
	    Connection con = null;
		TraduccionSeccionDAO tradSeccionDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradSeccionDAO = new TraduccionSeccionDAO(con);
			// Modificar la traducción
			tradSeccionDAO.modificar(tradSeccion);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public void eliminarTraduccionSeccion(String idIdioma, String idCodigo, String nivel) throws Exception {
	    Connection con = null;
		TraduccionSeccionDAO tradSeccionDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradSeccionDAO = new TraduccionSeccionDAO(con);
			// Eliminar el tradSeccion
			tradSeccionDAO.eliminar(idIdioma,Integer.parseInt(idCodigo),nivel);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public Collection listaTraduccionesSeccion(String idCodigo, String nivel) throws Exception {
		Connection con = null;
		TraduccionSeccionDAO tradSeccionDAO = null;
		Collection c = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradSeccionDAO = new TraduccionSeccionDAO(con);
			// Ejecuta la sentencia
			c = tradSeccionDAO.recuperarPorIdCodigoYIdNivel(Integer.parseInt(idCodigo),nivel);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return c;
	}
	
	public Collection listaSeccionesTraducidas(String idIdioma, String idCodigo) throws Exception {
		Connection con = null;
		TraduccionSeccionDAO tradSeccionDAO = null;
		Collection c = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradSeccionDAO = new TraduccionSeccionDAO(con);
			// Ejecuta la sentencia
			c = tradSeccionDAO.recuperarPorIdIdiomaYIdCodigo(idIdioma,Integer.parseInt(idCodigo));
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return c;
	}
	
	public Collection listaSeccionesTraducidasDelIndice(String idIdioma, String idCodigo) throws Exception {
		Connection con = null;
		TraduccionSeccionDAO tradSeccionDAO = null;
		Collection c = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradSeccionDAO = new TraduccionSeccionDAO(con);
			c = tradSeccionDAO.recuperarPorIdIdiomaYIdCodigoDelIndice(idIdioma,Integer.parseInt(idCodigo));
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return c;
	}

}
