/*
 * TraduccionTemaBO.java		03-ago-2005
 */
package codetic.bo;

import java.sql.Connection;
import java.util.*;

import javax.sql.DataSource;

import codetic.dao.*;
import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que contiene la lógica de negocio relacionada con la entidad TraduccionTema. 
 *
 * @author Miguel Angel Serna Mata
 */
public class TraduccionTemaBO {
	
	private DataSource dataSource;
	
	public TraduccionTemaBO(DataSource dataSource) {
		this.dataSource = dataSource;
	}
	
	public void crearTraduccionTema(TraduccionTemaDTO tradTema) throws Exception {
	    Connection con = null;
		TraduccionTemaDAO tradTemaDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradTemaDAO = new TraduccionTemaDAO(con);
			// Crear la traducción
			tradTemaDAO.crear(tradTema);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public TraduccionTemaDTO recuperarTraduccionTema(String idIdioma, String idTema) throws Exception {
	    Connection con = null;
		TraduccionTemaDAO tradTemaDAO = null;
		TraduccionTemaDTO tradTema = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradTemaDAO = new TraduccionTemaDAO(con);
			// Recuperar la traducción
			tradTema = tradTemaDAO.recuperar(idIdioma,Integer.parseInt(idTema));
			JDBCUtils.commit(con);
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return tradTema;
	}
	
	public void modificarTraduccionTema(TraduccionTemaDTO tradTema) throws Exception {
	    Connection con = null;
		TraduccionTemaDAO tradTemaDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradTemaDAO = new TraduccionTemaDAO(con);
			// Modificar la traducción
			tradTemaDAO.modificar(tradTema);
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public void eliminarTraduccionTema(String idIdioma, String idTema) throws Exception {
	    Connection con = null;
		TraduccionTemaDAO tradTemaDAO = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradTemaDAO = new TraduccionTemaDAO(con);
			// Eliminar el tradTema
			tradTemaDAO.eliminar(idIdioma,Integer.parseInt(idTema));
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
	}
	
	public Collection listaTraduccionesTema(String idTema) throws Exception {
		Connection con = null;
		TraduccionTemaDAO tradTemaDAO = null;
		Collection c = null;
		
		try {
		    con = JDBCUtils.getConnection(dataSource);
			tradTemaDAO = new TraduccionTemaDAO(con);
			// Ejecuta la sentencia
			c = tradTemaDAO.recuperarPorIdTema(Integer.parseInt(idTema));
			JDBCUtils.commit(con);
		} catch (DAOException e) {
		    JDBCUtils.rollback(con);
		    throw new BusinessException(e.getMessage());
		} catch (Exception e) {
		    JDBCUtils.rollback(con);
		    throw e;
		} finally {
		    JDBCUtils.close(con);
		}
		
		return c;
	}

}
