/*
 * AmbitoDAO.java		03-ago-2005
 */
package codetic.dao;

import java.sql.*;
import java.util.*;

import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que interactúa con la tabla Ambitos de la base de datos. 
 *
 * @author Miguel Angel Serna Mata
 */
public class AmbitoDAO {
	
	private Connection con;
	
	public AmbitoDAO(Connection con) {
		this.con = con;
	}
	
	public void crear(AmbitoDTO ambito) throws Exception{
	    if (recuperar(ambito.getIdAmbito()) != null) {
	     throw new DAOException("errors.dao.ambito.claveDuplicada");   
	    }
	    // Clave UNIQUE
	    if (recuperarPorNombreAdmon(ambito.getNombreAdmon()) != null) {
		     throw new DAOException("errors.dao.ambito.nombreAdmonDuplicado");   
		}
	    PreparedStatement ps = null;
		String sql = "INSERT INTO ambitos VALUES (?,?)";
		
	    try{
	        ps = con.prepareStatement(sql);
			ps.setInt(1,ambito.getIdAmbito());
			ps.setString(2,ambito.getNombreAdmon());
			ps.executeUpdate();
	    } catch (SQLException e) {
	        throw new RuntimeException(e.getMessage()); //("errors.dao.ambito.crear");
	    }
	    
	    JDBCUtils.close(ps);
	}
	
	public AmbitoDTO recuperar(int idAmbito) {
		PreparedStatement ps = null;
		ResultSet rs = null;
		AmbitoDTO ambito = null;
		String sql = "SELECT * FROM ambitos WHERE idAmbito=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setInt(1,idAmbito);
			rs = ps.executeQuery();   
			if (rs.next()) {
			    ambito = new AmbitoDTO();
			    ambito.setIdAmbito(rs.getInt("idAmbito"));
			    ambito.setNombreAdmon(rs.getString("nombreAdmon"));
			}
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.ambito.recuperar");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return(ambito);
	}
	
	public AmbitoDTO recuperarPorNombreAdmon(String nombreAdmon) {
		PreparedStatement ps = null;
		ResultSet rs = null;
		AmbitoDTO ambito = null;
		String sql = "SELECT * FROM ambitos WHERE nombreAdmon=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,nombreAdmon);
			rs = ps.executeQuery();   
			if (rs.next()) {
			    ambito = new AmbitoDTO();
			    ambito.setIdAmbito(rs.getInt("idAmbito"));
			    ambito.setNombreAdmon(rs.getString("nombreAdmon"));
			}
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.ambito.recuperar");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return(ambito);
	}
	
	public void modificar(AmbitoDTO ambito) throws Exception {
	    AmbitoDTO otroAmbito = null;
	    
	    if (recuperar(ambito.getIdAmbito()) == null) {
		     throw new DAOException("errors.dao.ambito.noExiste");   
		}
	    // Clave UNIQUE
	    otroAmbito = recuperarPorNombreAdmon(ambito.getNombreAdmon());
	    if ((otroAmbito != null) && (otroAmbito.getIdAmbito() != ambito.getIdAmbito())) {
		     throw new DAOException("errors.dao.ambito.nombreAdministracionDuplicado");   
		}
		PreparedStatement ps = null;
		String sql = "UPDATE ambitos SET nombreAdmon=? "+
		             "WHERE idAmbito=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,ambito.getNombreAdmon());
			ps.setInt(2,ambito.getIdAmbito());
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.ambito.modificar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public void eliminar(int idAmbito) throws Exception {
		PreparedStatement ps = null;
		String sql = "DELETE FROM ambitos WHERE idAmbito=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setInt(1,idAmbito);
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.ambito.eliminar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public Collection recuperarTodos() throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		AmbitoDTO ambito = null;
		String sql = "SELECT * FROM ambitos";
		
		try {
		    ps = con.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
				ambito = new AmbitoDTO();
				ambito.setIdAmbito(rs.getInt(1));
				ambito.setNombreAdmon(rs.getString(2));
				c.add(ambito);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.ambito.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}
	
	public Map recuperarTodosMap() throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Map m = new HashMap();
		AmbitoDTO ambito = null;
		String sql = "SELECT * FROM ambitos";
		
		try {
		    ps = con.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
				ambito = new AmbitoDTO();
				ambito.setIdAmbito(rs.getInt(1));
			    ambito.setNombreAdmon(rs.getString(2));
				m.put(new Integer(ambito.getIdAmbito()),ambito);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.ambito.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return m;
	}

}