/*
 * AsociacionDAO.java		03-ago-2005
 */
package codetic.dao;

import java.sql.*;
import java.util.*;

import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que interactúa con la tabla Asociaciones de la base de datos. 
 *
 * @author Miguel Angel Serna Mata
 */
public class AsociacionDAO {
	
	private Connection con;
	
	public AsociacionDAO(Connection con) {
		this.con = con;
	}
	
	public void crear(AsociacionDTO asociacion) throws Exception{
	    if (recuperar(asociacion.getIdAsociacion()) != null) {
	     throw new DAOException("errors.dao.asociacion.claveDuplicada");   
	    }
	    // Clave UNIQUE
	    if (recuperarPorNombre(asociacion.getNombre()) != null) {
		     throw new DAOException("errors.dao.asociacion.nombreDuplicado");   
		}
	    PreparedStatement ps = null;
		String sql = "INSERT INTO asociaciones VALUES (?,?,?,?,?,?)";
		
	    try{
	        ps = con.prepareStatement(sql);
			ps.setInt(1,asociacion.getIdAsociacion());
			ps.setInt(2,asociacion.getIdAmbito());
			ps.setString(3,asociacion.getNombre());
			ps.setString(4,asociacion.getSiglas());
			ps.setString(5,asociacion.getWeb());
			ps.setString(6,asociacion.getAnyoCreacion());
			ps.executeUpdate();
	    } catch (SQLException e) {
	        throw new RuntimeException("errors.dao.asociacion.crear");
	    }
	    
	    JDBCUtils.close(ps);
	}
	
	public AsociacionDTO recuperar(int idAsociacion) {
		PreparedStatement ps = null;
		ResultSet rs = null;
		AsociacionDTO asociacion = null;
		String sql = "SELECT * FROM asociaciones WHERE idAsociacion=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setInt(1,idAsociacion);
			rs = ps.executeQuery();   
			if (rs.next()) {
			    asociacion = new AsociacionDTO();
			    asociacion.setIdAsociacion(rs.getInt(1));
				asociacion.setIdAmbito(rs.getInt(2));
				asociacion.setNombre(rs.getString(3));
				asociacion.setSiglas(rs.getString(4));
				asociacion.setWeb(rs.getString(5));
				asociacion.setAnyoCreacion(rs.getString(6));
			}
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.asociacion.recuperar");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return(asociacion);
	}
	
	public void modificar(AsociacionDTO asociacion) throws Exception {
	    AsociacionDTO otraAsociacion = null;
	    
	    if (recuperar(asociacion.getIdAsociacion()) == null) {
		     throw new DAOException("errors.dao.asociacion.noExiste");   
		}
	    // Clave UNIQUE
	    otraAsociacion = recuperarPorNombre(asociacion.getNombre());
	    if ((otraAsociacion != null) && (otraAsociacion.getIdAsociacion() != asociacion.getIdAsociacion())) {
		     throw new DAOException("errors.dao.asociacion.nombreDuplicado");   
		}
		PreparedStatement ps = null;
		String sql = "UPDATE asociaciones SET idAmbito=?, nombre=?, siglas=?, web=?, anyoCreacion=?"+
		             "WHERE idAsociacion=?";
		
		try {
		    ps = con.prepareStatement(sql);
		    ps.setInt(1,asociacion.getIdAmbito());
			ps.setString(2,asociacion.getNombre());
			ps.setString(3,asociacion.getSiglas());
			ps.setString(4,asociacion.getWeb());
			ps.setString(5,asociacion.getAnyoCreacion());
			ps.setInt(6,asociacion.getIdAsociacion());
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.asociacion.modificar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public void eliminar(int idAsociacion) throws Exception {
		PreparedStatement ps = null;
		String sql = "DELETE FROM asociaciones WHERE idAsociacion=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setInt(1,idAsociacion);
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.asociacion.eliminar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public Collection recuperarTodos() throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		AsociacionDTO asociacion = null;
		String sql = "SELECT * FROM asociaciones";
		
		try {
		    ps = con.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
				asociacion = new AsociacionDTO();
			    asociacion.setIdAsociacion(rs.getInt(1));
				asociacion.setIdAmbito(rs.getInt(2));
				asociacion.setNombre(rs.getString(3));
				asociacion.setSiglas(rs.getString(4));
				asociacion.setWeb(rs.getString(5));
				asociacion.setAnyoCreacion(rs.getString(6));
				c.add(asociacion);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.asociacion.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}
	
	public Collection recuperarPorIdAmbito(int idAmbito) throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		AsociacionDTO asociacion = null;
		String sql = "SELECT * FROM asociaciones WHERE idAmbito=?";
		
		try {
		    ps = con.prepareStatement(sql);
		    ps.setInt(1,idAmbito);
			rs = ps.executeQuery();
			while (rs.next()) {
				asociacion = new AsociacionDTO();
			    asociacion.setIdAsociacion(rs.getInt(1));
				asociacion.setIdAmbito(rs.getInt(2));
				asociacion.setNombre(rs.getString(3));
				asociacion.setSiglas(rs.getString(4));
				asociacion.setWeb(rs.getString(5));
				asociacion.setAnyoCreacion(rs.getString(6));
				c.add(asociacion);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.asociacion.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}
	
	public AsociacionDTO recuperarPorNombre(String nombre) {
		PreparedStatement ps = null;
		ResultSet rs = null;
		AsociacionDTO asociacion = null;
		String sql = "SELECT * FROM asociaciones WHERE nombre=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,nombre);
			rs = ps.executeQuery();   
			if (rs.next()) {
			    asociacion = new AsociacionDTO();
			    asociacion.setIdAsociacion(rs.getInt(1));
				asociacion.setIdAmbito(rs.getInt(2));
				asociacion.setNombre(rs.getString(3));
				asociacion.setSiglas(rs.getString(4));
				asociacion.setWeb(rs.getString(5));
				asociacion.setAnyoCreacion(rs.getString(6));
			}
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.asociacion.recuperar");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return(asociacion);
	}

}