/*
 * BusquedaDAO.java		03-ago-2005
 */
package codetic.dao;

import java.sql.*;
import java.util.*;

import codetic.dto.*;
import codetic.utils.*;

/**
 * Clase que interactúa con varias tablas de la base de datos
 * para realizar búsquedas de artículos de códigos éticos. 
 *
 * @author Miguel Angel Serna Mata
 */
public class BusquedaDAO {
	
	private Connection con;
	private static final String OR = "OR";
	private static final String AND = "AND";
	private static final String SQL_TEMAS = "SELECT idTema "
	    + "FROM seccionesTemas st "
	    + "WHERE st.idCodigo = ts.idCodigo AND st.nivel = ts.nivel";
	private static final String SQL_GRUPOS = "SELECT idGrupoDeTemas "
    	+ "FROM seccionesTemas st, temas t "
    	+ "WHERE st.idCodigo = ts.idCodigo AND st.nivel = ts.nivel AND st.idTema = t.idTema";
	
	public BusquedaDAO(Connection con) {
		this.con = con;
	}
	
	public Collection recuperarResultados(BusquedaDTO busqueda) throws Exception {
 		PreparedStatement ps = null;
		ResultSet rs = null;
		ResultadoDTO resultado = null;
		String sql = ""; //"SELECT * FROM traduccionesSecciones WHERE";
		Collection c = new ArrayList();
		StringTokenizer st = null;
		int i;
		String palabra = null;
		String wPalabras = "";
		String wTitulo = null;
		String wTexto = null;
		String operador = null;
		String wIdiomas = "";
		String idiomas[] = null;
		String wListaIdiomas = null;
		String asociaciones[] = null;
		String select, from, where = null;
		String wAsociaciones = null;
		String wListaAsociaciones = null;
		String wReferencias = null;
		String wAmbitos = null;
		String[] ambitos = null;
		String wListaAmbitos = null;
		String[] temas = null;
		String wTemas = null;
		String wListaTemas = null;
		String[] gruposDeTemas = null;
		String wGruposDeTemas = null;
		String wListaGruposDeTemas = null;
		
		// JDBC PreparedStatement using LIKE clause
		// + http://archives.neohapsis.com/archives/dev/sapdb/2003-q4/0767.html
		// Mejorar el LIKE con RLIKE
		// + http://www.gadgetopia.com/post/2621
		
		// Inicializar los fragmentos de la sentencia SQL
		select = "SELECT DISTINCT ts.idCodigo, ts.idIdioma, ts.nivel, ts.titulo AS tituloSeccion, ts.seudotitulo AS seudotituloSeccion," +
				"texto, tc.titulo AS tituloCodigo, nombre AS nombreAsociacion, siglas AS siglasAsociacion";
		from = "FROM asociaciones a, codigos c, traduccionesCodigos tc, traduccionesSecciones ts";
		where = "WHERE ts.idCodigo = c.idCodigo AND c.idCodigo = tc.idCodigo AND c.idAsociacion = a.idAsociacion AND ts.idIdioma = tc.idIdioma " +
		        "AND ts.texto != ''"; // Para evitar mostrar secciones "vacías"
		// 1. Palabras
		// -----------
		if (! busqueda.getPalabras().equals("")) {
		    // 1.a. Frase exacta
		    if (busqueda.getCoincidenciaPalabras().equals(Ctes.EXACTA)) {
		        palabra = busqueda.getPalabras();
		        wTitulo = "(ts.titulo LIKE '%" + palabra + "%' AND ts.titulo RLIKE '[[:<:]]" + palabra + "[[:>:]]')";
		        wTexto = "(texto LIKE '%" + palabra + "%' AND texto RLIKE '[[:<:]]" + palabra + "[[:>:]]')";
		        wPalabras = "(" + wTitulo + " " + OR + " " + wTexto + ")";
		    } else {
		    // 1.b. Cualquiera (por defecto) y todas las palabras 
		        wPalabras = "";
		        if (busqueda.getCoincidenciaPalabras().equals(Ctes.TODAS)) {
		            operador = AND;
		        } else {
		            operador = OR;
		        }
		        i = 0;
		        st = new StringTokenizer(busqueda.getPalabras());
		        while (st.hasMoreTokens()) {
		            palabra = st.nextToken();
		            wTitulo = "(ts.titulo LIKE '%" + palabra + "%' AND ts.titulo RLIKE '[[:<:]]" + palabra + "[[:>:]]')";
		            wTexto = "(texto LIKE '%" + palabra + "%' AND texto RLIKE '[[:<:]]" + palabra + "[[:>:]]')";
		            if (i != 0) {
		                wPalabras = wPalabras + " " + operador + " (" + wTitulo + " " + OR + " " + wTexto + ")";
		            } else {
		                wPalabras = wPalabras + " (" + wTitulo + " " + OR + " " + wTexto + ")";
		            }
		            i++;
		        }
		    }
		    // Consulta SQL
			where += " " + AND + "(" + wPalabras + ")";
		}
		// 2. Idiomas
		// -------------
		if (busqueda.getIdiomas().length > 0) {
		    idiomas = busqueda.getIdiomas();
		    // Contruir la parte del WHERE
		    wListaIdiomas = "('" + idiomas[0];
		    for (int j=1; j < idiomas.length; j++) {
		        wListaIdiomas += "','" + idiomas[j];
		    }
		    wListaIdiomas += "')";
		    wIdiomas = "(ts.idIdioma IN " + wListaIdiomas + ")";
		    where += " " + AND + " " +  wIdiomas;
		}
		// 3. Asociaciones
		// ---------------
		if (busqueda.getAsociaciones().length > 0) {
		    asociaciones = busqueda.getAsociaciones();
		    // Contruir la parte del WHERE
		    wListaAsociaciones = "('" + asociaciones[0];
		    for (int j=1; j < asociaciones.length; j++) {
		        wListaAsociaciones += "','" + asociaciones[j];
		    }
		    wListaAsociaciones += "')";
		    wAsociaciones = "(c.idAsociacion IN " + wListaAsociaciones + ")";
		}
		// 4. Ámbitos
		// ---------------
		if (busqueda.getAmbitos().length > 0) {
		    ambitos = busqueda.getAmbitos();
		    // Contruir la parte del WHERE
		    wListaAmbitos = "('" + ambitos[0];
		    for (int j=1; j < ambitos.length; j++) {
		        wListaAmbitos += "','" + ambitos[j];
		    }
		    wListaAmbitos += "')";
		    wAmbitos = "(a.idAmbito IN " + wListaAmbitos + ")";
		}
		// 3-4. WHERE de Asociaciones y Ámbitos
		// ------------------------------------
		if ((busqueda.getAsociaciones().length > 0) && (busqueda.getAmbitos().length > 0)) {
		    where += " " + AND + " (" +  wAsociaciones + " " + OR + " " + wAmbitos + ")";
		} else if (busqueda.getAsociaciones().length > 0) {
		    where += " " + AND + " " + wAsociaciones;
		} else if (busqueda.getAmbitos().length > 0) {
		    where += " " + AND + " " + wAmbitos;
		}
		// 5. Temas
		// --------
		if (busqueda.getTemas().length > 0) {
		    temas = busqueda.getTemas();
		    // 5.1. Cualquier tema
		    // ------------------- 
		    if (busqueda.getCoincidenciaGruposYTemas().equals(Ctes.CUALQUIERA)) {
		        // Modificar el FROM
			    //from += ", seccionesTemas st";
			    // Contruir la parte del WHERE
		        wListaTemas = "('" + temas[0];
			    for (int j=1; j < temas.length; j++) {
			        wListaTemas += "','" + temas[j];
			    }
			    wListaTemas += "')";
			    wTemas = "(st.idTema IN " + wListaTemas + ")";
			    /*where += " " + AND + " st.idCodigo = ts.idCodigo"
			      + " " + AND + " st.nivel = ts.nivel"
			      + " " + AND + " " + wTemas;*/
		    }
		    // 5.2. Todos los temas
		    // --------------------
		    if (busqueda.getCoincidenciaGruposYTemas().equals(Ctes.TODOS)) {
		        // Contruir la parte del WHERE
		        wTemas = "";
		        wTemas += temas[0] + " IN (" + SQL_TEMAS +")";
			    for (int j=1; j < temas.length; j++) {
			        wTemas += " " + AND + " " + temas[j] + " IN (" + SQL_TEMAS +")";
			    }
			    //where += " " + wTemas;
		    }
		}
		// 6. Grupos de Temas
		// ------------------
		if (busqueda.getGruposDeTemas().length > 0) {
		    gruposDeTemas = busqueda.getGruposDeTemas();
		    // 6.1. Cualquier grupo
		    // --------------------
		    if (busqueda.getCoincidenciaGruposYTemas().equals(Ctes.CUALQUIERA)) {
		        // Modificar el FROM
			    //from += ", seccionesTemas st, temas t";
			    // Contruir la parte del WHERE
			    wListaGruposDeTemas = "('" + gruposDeTemas[0];
			    for (int j=1; j < gruposDeTemas.length; j++) {
			        wListaGruposDeTemas += "','" + gruposDeTemas[j];
			    }
			    wListaGruposDeTemas += "')";
			    wGruposDeTemas = "(t.idGrupoDeTemas IN " + wListaGruposDeTemas + ")";
			    /*where += " " + AND + " st.idCodigo = ts.idCodigo"
			      + " " + AND + " st.nivel = ts.nivel"
			      + " " + AND + " st.idTema = t.idTema"
			      + " " + AND + " " + wGruposDeTemas;*/
		    }
		    // 6.2. Todos los grupos
		    // ---------------------
		    if (busqueda.getCoincidenciaGruposYTemas().equals(Ctes.TODOS)) {
		        // Contruir la parte del WHERE
		        wGruposDeTemas = "";
		        wGruposDeTemas += gruposDeTemas[0] + " IN (" + SQL_GRUPOS +")";
			    for (int j=1; j < gruposDeTemas.length; j++) {
			        wGruposDeTemas += " " + AND + " " + gruposDeTemas[j] + " IN (" + SQL_GRUPOS + ")";
			    }
			    //where += " " + wGruposDeTemas;
		    }
		}
		// 5-6. FROM y WHERE de Temas y Grupos de Temas
		// --------------------------------------------
		if (busqueda.getCoincidenciaGruposYTemas().equals(Ctes.CUALQUIERA)) {
		    if ((busqueda.getTemas().length > 0) && (busqueda.getGruposDeTemas().length > 0)) {
		        from += ", seccionesTemas st, temas t";
			    where += " " + AND + " st.idCodigo = ts.idCodigo"
			      + " " + AND + " st.nivel = ts.nivel"
			      + " " + AND + " st.idTema = t.idTema"
			      + " " + AND + " (" +  wTemas + " " + OR + " " + wGruposDeTemas + ")";
			} else if (busqueda.getTemas().length > 0) {
			    from += ", seccionesTemas st";
			    where += " " + AND + " st.idCodigo = ts.idCodigo"
			      + " " + AND + " st.nivel = ts.nivel"
			      + " " + AND + " " + wTemas;
			} else if (busqueda.getGruposDeTemas().length > 0) {
			    from += ", seccionesTemas st, temas t";
			    where += " " + AND + " st.idCodigo = ts.idCodigo"
			      + " " + AND + " st.nivel = ts.nivel"
			      + " " + AND + " st.idTema = t.idTema"
			      + " " + AND + " " + wGruposDeTemas;
			}
		}
		if (busqueda.getCoincidenciaGruposYTemas().equals(Ctes.TODOS)) {
		    if ((busqueda.getTemas().length > 0) && (busqueda.getGruposDeTemas().length > 0)) {
			    where += " " + AND + " (" +  wTemas + " " + AND + " " + wGruposDeTemas + ")";
			} else if (busqueda.getTemas().length > 0) {
			    where += " " + AND + " " + wTemas;
			} else if (busqueda.getGruposDeTemas().length > 0) {
			    where += " " + AND + " " + wGruposDeTemas;
			}
		}
		// Reconstruir la consulta sql
		sql = select + " " + from + " " + where;
		try {
		    ps = con.prepareStatement(sql);
			rs = ps.executeQuery();   
			while (rs.next()) {
			    resultado = new ResultadoDTO();
			    resultado.setIdIdioma(rs.getString("idIdioma"));
			    resultado.setIdCodigo(rs.getInt("idCodigo"));
			    resultado.setNivel(rs.getString("nivel"));
			    resultado.setTituloSeccion(rs.getString("tituloSeccion"));
			    resultado.setSeudotituloSeccion(rs.getString("seudotituloSeccion"));
			    resultado.setTexto(rs.getString("texto"));
			    resultado.setTituloCodigo(rs.getString("tituloCodigo")); 
                resultado.setNombreAsociacion(rs.getString("nombreAsociacion"));
                resultado.setSiglasAsociacion(rs.getString("siglasAsociacion"));
			    c.add(resultado);
			}
		} catch (SQLException e) {
	        throw new RuntimeException(e.getMessage()); /*"errors.dao.busqueda");*/
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}
}