/*
 * CodigoDAO.java		03-ago-2005
 */
package codetic.dao;

import java.sql.*;
import java.util.*;

import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que interactúa con la tabla Codigos de la base de datos. 
 *
 * @author Miguel Angel Serna Mata
 */
public class CodigoDAO {
	
	private Connection con;
	
	public CodigoDAO(Connection con) {
		this.con = con;
	}
	
	public void crear(CodigoDTO codigo) throws DAOException{
	    if (recuperar(codigo.getIdCodigo()) != null) {
	     throw new DAOException("errors.dao.codigo.claveDuplicada");   
	    }
	    // Clave UNIQUE
	    if (recuperarPorIdAsociacionYTituloOriginal(codigo.getIdAsociacion(),codigo.getTituloOriginal()) != null) {
		     throw new DAOException("errors.dao.codigo.tituloOriginalDuplicado");   
		}
	    PreparedStatement ps = null;
		String sql = "INSERT INTO codigos VALUES (?,?,?,?,?)";
		
	    try{
	        ps = con.prepareStatement(sql);
			ps.setInt(1,codigo.getIdCodigo());
			ps.setInt(2,codigo.getIdAsociacion());
			ps.setString(3,codigo.getTituloOriginal());
			ps.setString(4,codigo.getWeb());
			ps.setString(5,codigo.getAnyoAprobacion());
			ps.executeUpdate();
	    } catch (SQLException e) {
	        throw new RuntimeException("errors.dao.codigo.crear");
	    }
	    
	    JDBCUtils.close(ps);
	}
	
	public CodigoDTO recuperar(int idCodigo) {
		PreparedStatement ps = null;
		ResultSet rs = null;
		CodigoDTO codigo = null;
		String sql = "SELECT * FROM codigos WHERE idCodigo=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setInt(1,idCodigo);
			rs = ps.executeQuery();   
			if (rs.next()) {
			    codigo = new CodigoDTO();
			    codigo.setIdCodigo(rs.getInt(1));
				codigo.setIdAsociacion(rs.getInt(2));
				codigo.setTituloOriginal(rs.getString(3));
				codigo.setWeb(rs.getString(4));
				codigo.setAnyoAprobacion(rs.getString(5));
			}
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.codigo.recuperar");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return(codigo);
	}
	
	public void modificar(CodigoDTO codigo) throws Exception {
	    CodigoDTO otroCodigo = null;
	    
	    if (recuperar(codigo.getIdCodigo()) == null) {
		     throw new DAOException("errors.dao.codigo.noExiste");   
		}
	    // Clave UNIQUE
	    otroCodigo = recuperarPorIdAsociacionYTituloOriginal(codigo.getIdAsociacion(),codigo.getTituloOriginal());
	    if ((otroCodigo != null) && (otroCodigo.getIdCodigo() != codigo.getIdCodigo())) {
		     throw new DAOException("errors.dao.codigo.tituloOriginalDuplicado");   
		}
		PreparedStatement ps = null;
		String sql = "UPDATE codigos SET tituloOriginal=?, web=?, anyoAprobacion=?"+
		             "WHERE idCodigo=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,codigo.getTituloOriginal());
			ps.setString(2,codigo.getWeb());
			ps.setString(3,codigo.getAnyoAprobacion());
			ps.setInt(4,codigo.getIdCodigo());
			//ps.setInt(5,codigo.getIdAsociacion());
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.codigo.modificar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public void eliminar(int idCodigo) throws Exception {
		PreparedStatement ps = null;
		String sql = "DELETE FROM codigos WHERE idCodigo=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setInt(1,idCodigo);
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.codigo.eliminar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public Collection recuperarTodos() throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		CodigoDTO codigo = null;
		String sql = "SELECT * FROM codigos";
		
		try {
		    ps = con.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
				codigo = new CodigoDTO();
				codigo.setIdCodigo(rs.getInt(1));
				codigo.setIdAsociacion(rs.getInt(2));
				codigo.setTituloOriginal(rs.getString(3));
				codigo.setWeb(rs.getString(4));
				codigo.setAnyoAprobacion(rs.getString(5));
				c.add(codigo);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.codigo.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}
	
	public Collection recuperarPorIdAsociacion(int idAsociacion) throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		CodigoDTO codigo = null;
		String sql = "SELECT * FROM codigos WHERE idAsociacion=?";
		
		try {
		    ps = con.prepareStatement(sql);
		    ps.setInt(1,idAsociacion);
			rs = ps.executeQuery();
			while (rs.next()) {
				codigo = new CodigoDTO();
				codigo.setIdCodigo(rs.getInt(1));
				codigo.setIdAsociacion(rs.getInt(2));
				codigo.setTituloOriginal(rs.getString(3));
				codigo.setWeb(rs.getString(4));
				codigo.setAnyoAprobacion(rs.getString(5));
				c.add(codigo);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.codigo.recuperar");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}
	
	public CodigoDTO recuperarPorIdAsociacionYTituloOriginal(int idAsociacion, String tituloOriginal) {
		PreparedStatement ps = null;
		ResultSet rs = null;
		CodigoDTO codigo = null;
		String sql = "SELECT * FROM codigos WHERE idAsociacion=? AND tituloOriginal=?";
		
		try {
		    ps = con.prepareStatement(sql);
		    ps.setInt(1,idAsociacion);
		    ps.setString(2,tituloOriginal);
			rs = ps.executeQuery();
			if (rs.next()) {
			    codigo = new CodigoDTO();
			    codigo.setIdCodigo(rs.getInt(1));
				codigo.setIdAsociacion(rs.getInt(2));
				codigo.setTituloOriginal(rs.getString(3));
				codigo.setWeb(rs.getString(4));
				codigo.setAnyoAprobacion(rs.getString(5));
			}   
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.codigo.recuperar");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return codigo;
	}

}