/*
 * GrupoDeTemasDAO.java		03-ago-2005
 */
package codetic.dao;

import java.sql.*;
import java.util.*;

import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que interactúa con la tabla GruposDeTemas de la base de datos. 
 *
 * @author Miguel Angel Serna Mata
 */
public class GrupoDeTemasDAO {
	
	private Connection con;
	
	public GrupoDeTemasDAO(Connection con) {
		this.con = con;
	}
	
	public void crear(GrupoDeTemasDTO grupoDeTemas) throws Exception{
	    if (recuperar(grupoDeTemas.getIdGrupoDeTemas()) != null) {
	     throw new DAOException("errors.dao.grupoDeTemas.claveDuplicada");   
	    }
	    // Clave UNIQUE
	    if (recuperarPorNombreAdmon(grupoDeTemas.getNombreAdmon()) != null) {
		     throw new DAOException("errors.dao.grupoDeTemas.nombreAdministracionDuplicado");   
		}
	    PreparedStatement ps = null;
		String sql = "INSERT INTO gruposDeTemas VALUES (?,?)";
		
	    try{
	        ps = con.prepareStatement(sql);
			ps.setInt(1,grupoDeTemas.getIdGrupoDeTemas());
			ps.setString(2,grupoDeTemas.getNombreAdmon());
			ps.executeUpdate();
	    } catch (SQLException e) {
	        throw new RuntimeException("errors.dao.grupoDeTemas.crear");
	    }
	    
	    JDBCUtils.close(ps);
	}
	
	public GrupoDeTemasDTO recuperar(int idGrupoDeTemas) {
		PreparedStatement ps = null;
		ResultSet rs = null;
		GrupoDeTemasDTO grupoDeTemas = null;
		String sql = "SELECT * FROM gruposDeTemas WHERE idGrupoDeTemas=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setInt(1,idGrupoDeTemas);
			rs = ps.executeQuery();   
			if (rs.next()) {
			    grupoDeTemas = new GrupoDeTemasDTO();
			    grupoDeTemas.setIdGrupoDeTemas(rs.getInt(1));
			    grupoDeTemas.setNombreAdmon(rs.getString(2));
			}
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.grupoDeTemas.recuperar");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return(grupoDeTemas);
	}
	
	public void modificar(GrupoDeTemasDTO grupoDeTemas) throws Exception {
	    GrupoDeTemasDTO otroGrupoDeTemas = null;
	    if (recuperar(grupoDeTemas.getIdGrupoDeTemas()) == null) {
		     throw new DAOException("errors.dao.grupoDeTemas.noExiste");   
		}
	    // Clave UNIQUE
	    otroGrupoDeTemas = recuperarPorNombreAdmon(grupoDeTemas.getNombreAdmon());
	    if ((otroGrupoDeTemas != null) && (otroGrupoDeTemas.getIdGrupoDeTemas() != grupoDeTemas.getIdGrupoDeTemas())) {
		     throw new DAOException("errors.dao.grupoDeTemas.nombreAdministracionDuplicado");   
		}
		PreparedStatement ps = null;
		String sql = "UPDATE gruposDeTemas SET nombreAdmon=?"+
		             "WHERE idGrupoDeTemas=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,grupoDeTemas.getNombreAdmon());
			ps.setInt(2,grupoDeTemas.getIdGrupoDeTemas());
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.grupoDeTemas.modificar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public void eliminar(int idGrupoDeTemas) throws Exception {
		PreparedStatement ps = null;
		String sql = "DELETE FROM gruposDeTemas WHERE idGrupoDeTemas=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setInt(1,idGrupoDeTemas);
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.grupoDeTemas.eliminar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public Collection recuperarTodos() throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		GrupoDeTemasDTO grupoDeTemas = null;
		String sql = "SELECT * FROM gruposDeTemas";
		
		try {
		    ps = con.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
				grupoDeTemas = new GrupoDeTemasDTO();
				grupoDeTemas.setIdGrupoDeTemas(rs.getInt(1));
			    grupoDeTemas.setNombreAdmon(rs.getString(2));
				c.add(grupoDeTemas);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.grupoDeTemas.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}
	
	public Map recuperarTodosMap() throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Map m = new HashMap();
		GrupoDeTemasDTO grupoDeTemas = null;
		String sql = "SELECT * FROM gruposDeTemas";
		
		try {
		    ps = con.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
				grupoDeTemas = new GrupoDeTemasDTO();
				grupoDeTemas.setIdGrupoDeTemas(rs.getInt(1));
			    grupoDeTemas.setNombreAdmon(rs.getString(2));
				m.put(new Integer(grupoDeTemas.getIdGrupoDeTemas()),grupoDeTemas);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.grupoDeTemas.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return m;
	}
	
	public GrupoDeTemasDTO recuperarPorNombreAdmon(String nombreAdmon) {
		PreparedStatement ps = null;
		ResultSet rs = null;
		GrupoDeTemasDTO grupoDeTemas = null;
		String sql = "SELECT * FROM gruposDeTemas WHERE nombreAdmon=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,nombreAdmon);
			rs = ps.executeQuery();   
			if (rs.next()) {
			    grupoDeTemas = new GrupoDeTemasDTO();
			    grupoDeTemas.setIdGrupoDeTemas(rs.getInt(1));
			    grupoDeTemas.setNombreAdmon(rs.getString(2));
			}
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.grupoDeTemas.recuperar");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return(grupoDeTemas);
	}

}