/*
 * IdiomaDAO.java		02-ago-2005
 */
package codetic.dao;

import java.sql.*;
import java.util.*;

import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que interactúa con la tabla Idiomas de la base de datos. 
 *
 * @author Miguel Angel Serna Mata
 */
public class IdiomaDAO {
	
	private Connection con;
	
	public IdiomaDAO(Connection con) {
		this.con = con;
	}
	
	public void crear(IdiomaDTO idioma) throws DAOException{
	    if (recuperar(idioma.getIdIdioma()) != null) {
	     throw new DAOException("errors.dao.idioma.claveDuplicada");   
	    }
	    // Clave UNIQUE
	    if (recuperarPorNombre(idioma.getNombre()) != null) {
		     throw new DAOException("errors.dao.idioma.nombreDuplicado");   
		}
	    PreparedStatement ps = null;
		String sql = "INSERT INTO idiomas VALUES (?,?)";
		
	    try{
	        ps = con.prepareStatement(sql);
			ps.setString(1,idioma.getIdIdioma());
			ps.setString(2,idioma.getNombre());
			ps.executeUpdate();
	    } catch (SQLException e) {
	        throw new RuntimeException("errors.dao.idioma.crear");
	    }
	    
	    JDBCUtils.close(ps);
	}
	
	public IdiomaDTO recuperar(String idIdioma) {
		PreparedStatement ps = null;
		ResultSet rs = null;
		IdiomaDTO idioma = null;
		String sql = "SELECT * FROM idiomas WHERE idIdioma=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,idIdioma);
			rs = ps.executeQuery();   
			if (rs.next()) {
			    idioma = new IdiomaDTO();
			    idioma.setIdIdioma(rs.getString("idIdioma"));
			    idioma.setNombre(rs.getString("nombre"));
			}
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.idioma.recuperar");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return(idioma);
	}
	
	public void modificar(IdiomaDTO idioma) throws Exception {
	    IdiomaDTO otroIdioma = null;
	    
	    if (recuperar(idioma.getIdIdioma()) == null) {
		     throw new DAOException("errors.dao.idioma.noExiste");   
		}
	    // Clave UNIQUE
	    otroIdioma = recuperarPorNombre(idioma.getNombre());
	    if ((otroIdioma != null) && (otroIdioma.getIdIdioma() != idioma.getIdIdioma())) {
		     throw new DAOException("errors.dao.idioma.nombreDuplicado");   
		}
		PreparedStatement ps = null;
		String sql = "UPDATE idiomas SET nombre=?"+
		             "WHERE idIdioma=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,idioma.getNombre());
			ps.setString(2,idioma.getIdIdioma());
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.idioma.modificar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public void eliminar(String idIdioma) throws Exception {
		PreparedStatement ps = null;
		String sql = "DELETE FROM idiomas WHERE idIdioma=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,idIdioma);
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.idioma.eliminar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public Collection recuperarTodos() throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		IdiomaDTO idioma = null;
		String sql = "SELECT * FROM idiomas";
		
		try {
		    ps = con.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
			    idioma = new IdiomaDTO();
				idioma.setIdIdioma(rs.getString(1));
				idioma.setNombre(rs.getString(2));
				c.add(idioma);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.idioma.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}
	
	public IdiomaDTO recuperarPorNombre(String nombre) {
		PreparedStatement ps = null;
		ResultSet rs = null;
		IdiomaDTO idioma = null;
		String sql = "SELECT * FROM idiomas WHERE nombre=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,nombre);
			rs = ps.executeQuery();   
			if (rs.next()) {
			    idioma = new IdiomaDTO();
			    idioma.setIdIdioma(rs.getString("idIdioma"));
			    idioma.setNombre(rs.getString("nombre"));
			}
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.idioma.recuperar");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return(idioma);
	}

}

