/*
 * SeccionDAO.java		03-ago-2005
 */
package codetic.dao;

import java.sql.*;
import java.util.*;

import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que interactúa con la tabla Secciones de la base de datos. 
 *
 * @author Miguel Angel Serna Mata
 */
public class SeccionDAO {
	
	private Connection con;
	
	public SeccionDAO(Connection con) {
		this.con = con;
	}
	
	public void crear(SeccionDTO seccion) throws Exception{
	    if (recuperar(seccion.getNivel(),seccion.getIdCodigo()) != null) {
	     throw new DAOException("errors.dao.seccion.claveDuplicada");   
	    }
	    PreparedStatement ps = null;
		String sql = "INSERT INTO secciones VALUES (?,?,?)";
		
	    try{
	        ps = con.prepareStatement(sql);
			ps.setString(1,seccion.getNivel());
			ps.setInt(2,seccion.getIdCodigo());
			ps.setString(3,seccion.getNombreAdmon());
			ps.executeUpdate();
	    } catch (SQLException e) {
	        throw new RuntimeException("errors.dao.seccion.crear");
	    }
	    
	    JDBCUtils.close(ps);
	}
	
	public SeccionDTO recuperar(String nivel, int idCodigo) {
		PreparedStatement ps = null;
		ResultSet rs = null;
		SeccionDTO seccion = null;
		String sql = "SELECT * FROM secciones WHERE nivel=? AND idCodigo=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,nivel);
			ps.setInt(2,idCodigo);
			rs = ps.executeQuery();   
			if (rs.next()) {
			    seccion = new SeccionDTO();
			    seccion.setNivel(rs.getString(1));
			    seccion.setIdCodigo(rs.getInt(2));
			    seccion.setNombreAdmon(rs.getString(3));
			}
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.seccion.recuperar");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return(seccion);
	}
	
	public void modificar(SeccionDTO seccion) throws Exception {
	    if (recuperar(seccion.getNivel(),seccion.getIdCodigo()) == null) {
		     throw new DAOException("errors.dao.seccion.noExiste");   
		}
		PreparedStatement ps = null;
		String sql = "UPDATE secciones SET nivel=?, nombreAdmon=?"+
		             "WHERE nivel=? AND idCodigo=?";
		
		try {
		    ps = con.prepareStatement(sql);
		    ps.setString(1,seccion.getNivel());
			ps.setString(2,seccion.getNombreAdmon());
			ps.setString(3,seccion.getNivel());
			ps.setInt(4,seccion.getIdCodigo());
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.seccion.modificar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public void modificar(SeccionDTO seccion, String nivelAntiguo) throws Exception {
	    if (recuperar(nivelAntiguo,seccion.getIdCodigo()) == null) {
		     throw new DAOException("errors.dao.seccion.noExiste");   
		}
	    if ((! seccion.getNivel().equals(nivelAntiguo)) && (recuperar(seccion.getNivel(),seccion.getIdCodigo()) != null)) {
		     throw new DAOException("errors.dao.seccion.claveDuplicada");   
		}
		PreparedStatement ps = null;
		String sql = "UPDATE secciones SET nivel=?, nombreAdmon=?"+
		             "WHERE nivel=? AND idCodigo=?";
		
		try {
		    ps = con.prepareStatement(sql);
		    ps.setString(1,seccion.getNivel());
			ps.setString(2,seccion.getNombreAdmon());
			ps.setString(3,nivelAntiguo);
			ps.setInt(4,seccion.getIdCodigo());
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.seccion.modificar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public void eliminar(String nivel, int idCodigo) throws Exception {
		PreparedStatement ps = null;
		String sql = "DELETE FROM secciones WHERE nivel=? AND idCodigo=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,nivel);
			ps.setInt(2,idCodigo);
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.seccion.eliminar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public Collection recuperarTodos() throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		SeccionDTO seccion = null;
		String sql = "SELECT * FROM secciones";
		
		try {
		    ps = con.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
				seccion = new SeccionDTO();
				seccion.setNivel(rs.getString(1));
			    seccion.setIdCodigo(rs.getInt(2));
			    seccion.setNombreAdmon(rs.getString(3));
				c.add(seccion);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.seccion.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}
	
	public Collection recuperarPorIdCodigo(int idCodigo) throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		SeccionDTO seccion = null;
		String sql = "SELECT * FROM secciones WHERE idCodigo=?";
		
		try {
		    ps = con.prepareStatement(sql);
		    ps.setInt(1,idCodigo);
			rs = ps.executeQuery();
			while (rs.next()) {
				seccion = new SeccionDTO();
				seccion.setNivel(rs.getString(1));
			    seccion.setIdCodigo(rs.getInt(2));
			    seccion.setNombreAdmon(rs.getString(3));
				c.add(seccion);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.seccion.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}

}