/*
 * SeccionTemaDAO.java		03-ago-2005
 */
package codetic.dao;

import java.sql.*;
import java.util.*;

import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que interactúa con la tabla SeccionesTemas de la base de datos. 
 *
 * @author Miguel Angel Serna Mata
 */
public class SeccionTemaDAO {
	
	private Connection con;
	
	public SeccionTemaDAO(Connection con) {
		this.con = con;
	}
	
	public void crear(SeccionTemaDTO seccionTema) throws Exception{
	    if (recuperar(seccionTema.getNivel(),seccionTema.getIdCodigo(),seccionTema.getIdTema()) != null) {
	     throw new DAOException("errors.dao.seccionTema.claveDuplicada");   
	    }
	    PreparedStatement ps = null;
		String sql = "INSERT INTO seccionesTemas VALUES (?,?,?)";
		
	    try{
	        ps = con.prepareStatement(sql);
			ps.setString(1,seccionTema.getNivel());
			ps.setInt(2,seccionTema.getIdCodigo());
			ps.setInt(3,seccionTema.getIdTema());
			ps.executeUpdate();
	    } catch (SQLException e) {
	        throw new RuntimeException("errors.dao.seccionTema.crear");
	    }
	    
	    JDBCUtils.close(ps);
	}
	
	public SeccionTemaDTO recuperar(String nivel, int idCodigo, int idTema) {
		PreparedStatement ps = null;
		ResultSet rs = null;
		SeccionTemaDTO seccionTema = null;
		String sql = "SELECT * FROM seccionesTemas WHERE nivel=? AND idCodigo=? AND idTema=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,nivel);
			ps.setInt(2,idCodigo);
			ps.setInt(3,idTema);
			rs = ps.executeQuery();   
			if (rs.next()) {
			    seccionTema = new SeccionTemaDTO();
			    seccionTema.setNivel(rs.getString(1));
			    seccionTema.setIdCodigo(rs.getInt(2));
			    seccionTema.setIdTema(rs.getInt(3));
			}
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.seccionTema.recuperar");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return(seccionTema);
	}
	
	/*
	 * En un principio no lo utilizaremos.
	 */
	public void modificar(SeccionTemaDTO seccionTema) throws Exception {
	    if (recuperar(seccionTema.getNivel(),seccionTema.getIdCodigo(),seccionTema.getIdTema()) == null) {
		     throw new DAOException("errors.dao.seccionTema.noExiste");   
		}
		PreparedStatement ps = null;
		String sql = "UPDATE seccionesTemas SET idTema=?"+
		             "WHERE nivel=? AND idCodigo=? AND idTema=?";
		
		try {
		    ps = con.prepareStatement(sql);
		    ps.setInt(1,seccionTema.getIdTema());
			ps.setString(2,seccionTema.getNivel());
			ps.setInt(3,seccionTema.getIdCodigo());
			ps.setInt(4,seccionTema.getIdTema());
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.seccionTema.modificar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public void eliminar(String nivel, int idCodigo, int idTema) throws Exception {
		PreparedStatement ps = null;
		String sql = "DELETE FROM seccionesTemas WHERE nivel=? AND idCodigo=? AND idTema=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,nivel);
			ps.setInt(2,idCodigo);
			ps.setInt(3,idTema);
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.seccionTema.eliminar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public void eliminarPorNivelYIdCodigo(String nivel, int idCodigo) throws Exception {
		PreparedStatement ps = null;
		String sql = "DELETE FROM seccionesTemas WHERE nivel=? AND idCodigo=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,nivel);
			ps.setInt(2,idCodigo);
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.seccionTema.eliminar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public Collection recuperarTodos() throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		SeccionTemaDTO seccionTema = null;
		String sql = "SELECT * FROM seccionesTemas";
		
		try {
		    ps = con.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
				seccionTema = new SeccionTemaDTO();
				seccionTema.setNivel(rs.getString(1));
			    seccionTema.setIdCodigo(rs.getInt(2));
			    seccionTema.setIdTema(rs.getInt(3));
				c.add(seccionTema);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.seccionTema.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}
	
	public Collection recuperarPorNivelYIdCodigo(String nivel, int idCodigo) throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		SeccionTemaDTO seccionTema = null;
		String sql = "SELECT * FROM seccionesTemas WHERE nivel=? AND idCodigo=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,nivel);
			ps.setInt(2,idCodigo);
			rs = ps.executeQuery();   
			while (rs.next()) {
				seccionTema = new SeccionTemaDTO();
				seccionTema.setNivel(rs.getString(1));
			    seccionTema.setIdCodigo(rs.getInt(2));
			    seccionTema.setIdTema(rs.getInt(3));
				c.add(seccionTema);
			} 
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.seccionTema.recuperar");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}
	
	public Collection recuperarPorIdTema(int idTema) {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		SeccionTemaDTO seccionTema = null;
		String sql = "SELECT * FROM seccionesTemas WHERE idTema=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setInt(1,idTema);
			rs = ps.executeQuery();   
			while (rs.next()) {
				seccionTema = new SeccionTemaDTO();
				seccionTema.setNivel(rs.getString(1));
			    seccionTema.setIdCodigo(rs.getInt(2));
			    seccionTema.setIdTema(rs.getInt(3));
				c.add(seccionTema);
			} 
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.seccionTema.recuperar");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}

}