/*
 * TemaDAO.java		03-ago-2005
 */
package codetic.dao;

import java.sql.*;
import java.util.*;

import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que interactúa con la tabla Temas de la base de datos. 
 *
 * @author Miguel Angel Serna Mata
 */
public class TemaDAO {
	
	private Connection con;
	
	public TemaDAO(Connection con) {
		this.con = con;
	}
	
	public void crear(TemaDTO tema) throws Exception{
	    if (recuperar(tema.getIdTema()) != null) {
	     throw new DAOException("errors.dao.tema.claveDuplicada");   
	    }
	    // Clave UNIQUE
	    if (recuperarPorNombreAdmon(tema.getNombreAdmon()) != null) {
		     throw new DAOException("errors.dao.tema.nombreAdministracionDuplicado");   
		}
	    PreparedStatement ps = null;
		String sql = "INSERT INTO temas VALUES (?,?,?)";
		
	    try{
	        ps = con.prepareStatement(sql);
			ps.setInt(1,tema.getIdTema());
			ps.setInt(2,tema.getIdGrupoDeTemas());
			ps.setString(3,tema.getNombreAdmon());
			ps.executeUpdate();
	    } catch (SQLException e) {
	        throw new RuntimeException("errors.dao.tema.crear");
	    }
	    
	    JDBCUtils.close(ps);
	}
	
	public TemaDTO recuperar(int idTema) {
		PreparedStatement ps = null;
		ResultSet rs = null;
		TemaDTO tema = null;
		String sql = "SELECT * FROM temas WHERE idTema=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setInt(1,idTema);
			rs = ps.executeQuery();   
			if (rs.next()) {
			    tema = new TemaDTO();
			    tema.setIdTema(rs.getInt(1));
			    tema.setIdGrupoDeTemas(rs.getInt(2));
			    tema.setNombreAdmon(rs.getString(3));
			}
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.tema.recuperar");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return(tema);
	}
	
	public void modificar(TemaDTO tema) throws Exception {
	    TemaDTO otroTema = null;
	    
	    if (recuperar(tema.getIdTema()) == null) {
		     throw new DAOException("errors.dao.tema.noExiste");   
		}
	    // Clave UNIQUE
	    otroTema = recuperarPorNombreAdmon(tema.getNombreAdmon());
	    if ((otroTema != null) && (otroTema.getIdTema() != tema.getIdTema())) {
		     throw new DAOException("errors.dao.tema.nombreAdministracionDuplicado");   
		}
		PreparedStatement ps = null;
		String sql = "UPDATE temas SET idGrupoDeTemas=?, nombreAdmon=?"+
		             "WHERE idTema=?";
		
		try {
		    ps = con.prepareStatement(sql);
		    ps.setInt(1,tema.getIdGrupoDeTemas());
			ps.setString(2,tema.getNombreAdmon());
			ps.setInt(3,tema.getIdTema());
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.tema.modificar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public void eliminar(int idTema) throws Exception {
		PreparedStatement ps = null;
		String sql = "DELETE FROM temas WHERE idTema=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setInt(1,idTema);
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.tema.eliminar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public Collection recuperarTodos() throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		TemaDTO tema = null;
		String sql = "SELECT * FROM temas";
		
		try {
		    ps = con.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
				tema = new TemaDTO();
				tema.setIdTema(rs.getInt(1));
			    tema.setIdGrupoDeTemas(rs.getInt(2));
			    tema.setNombreAdmon(rs.getString(3));
				c.add(tema);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.tema.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}
	
	public Collection recuperarPorIdGrupoDeTemas(int idGrupoDeTemas) throws Exception{
		PreparedStatement ps = null;
		ResultSet rs = null;
		TemaDTO tema = null;
		Collection c = new ArrayList();
		String sql = "SELECT * FROM temas WHERE idGrupoDeTemas=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setInt(1,idGrupoDeTemas);
			rs = ps.executeQuery();   
			while (rs.next()) {
				tema = new TemaDTO();
				tema.setIdTema(rs.getInt(1));
			    tema.setIdGrupoDeTemas(rs.getInt(2));
			    tema.setNombreAdmon(rs.getString(3));
				c.add(tema);
			}  
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.tema.recuperar");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}
	
	public TemaDTO recuperarPorNombreAdmon(String nombreAdmon) {
		PreparedStatement ps = null;
		ResultSet rs = null;
		TemaDTO tema = null;
		String sql = "SELECT * FROM temas WHERE nombreAdmon=?";
		
		try {
		    ps = con.prepareStatement(sql);
		    ps.setString(1,nombreAdmon);
			rs = ps.executeQuery();
			if (rs.next()) {
			    tema = new TemaDTO();
			    tema.setIdTema(rs.getInt(1));
			    tema.setIdGrupoDeTemas(rs.getInt(2));
			    tema.setNombreAdmon(rs.getString(3));
			}
		} catch (SQLException e) {
	        throw new RuntimeException(e.getMessage()); //"errors.dao.tema.recuperar");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return(tema);
	}

}