/*
 * TraduccionAmbitoDAO.java		03-ago-2005
 */
package codetic.dao;

import java.sql.*;
import java.util.*;

import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que interactúa con la tabla TraduccionesAmbitos de la base de datos. 
 *
 * @author Miguel Angel Serna Mata
 */
public class TraduccionAmbitoDAO {
	
	private Connection con;
	
	public TraduccionAmbitoDAO(Connection con) {
		this.con = con;
	}
	
	public void crear(TraduccionAmbitoDTO tradAmbito) throws DAOException{
	    if (recuperar(tradAmbito.getIdIdioma(),tradAmbito.getIdAmbito()) != null) {
	     throw new DAOException("errors.dao.traduccionAmbito.claveDuplicada");   
	    }
	    PreparedStatement ps = null;
		String sql = "INSERT INTO traduccionesAmbitos VALUES (?,?,?,?)";
		
	    try{
	        ps = con.prepareStatement(sql);
			ps.setString(1,tradAmbito.getIdIdioma());
			ps.setInt(2,tradAmbito.getIdAmbito());
			ps.setString(3,tradAmbito.getNombre());
			ps.setString(4,tradAmbito.getDescripcion());
			ps.executeUpdate();
	    } catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionAmbito.crear");
	    }
	    
	    JDBCUtils.close(ps);
	}
	
	public TraduccionAmbitoDTO recuperar(String idIdioma, int idAmbito) {
		PreparedStatement ps = null;
		ResultSet rs = null;
		TraduccionAmbitoDTO tradAmbito = null;
		String sql = "SELECT * FROM traduccionesAmbitos WHERE idIdioma=? AND idAmbito=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,idIdioma);
			ps.setInt(2,idAmbito);
			rs = ps.executeQuery();   
			if (rs.next()) {
			    tradAmbito = new TraduccionAmbitoDTO();
			    tradAmbito.setIdIdioma(rs.getString("idIdioma"));
			    tradAmbito.setIdAmbito(rs.getInt("idAmbito"));
			    tradAmbito.setNombre(rs.getString("nombre"));
			    tradAmbito.setDescripcion(rs.getString("descripcion"));
			}
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionAmbito.recuperar");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return(tradAmbito);
	}
	
	public void modificar(TraduccionAmbitoDTO tradAmbito) throws Exception {
	    if (recuperar(tradAmbito.getIdIdioma(),tradAmbito.getIdAmbito()) == null) {
		     throw new DAOException("errors.dao.traduccionAmbito.noExiste");   
	    }
		PreparedStatement ps = null;
		String sql = "UPDATE traduccionesAmbitos SET nombre=?, descripcion=?"+
		             "WHERE idIdioma=? AND idAmbito=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,tradAmbito.getNombre());
			ps.setString(2,tradAmbito.getDescripcion());
			ps.setString(3,tradAmbito.getIdIdioma());
			ps.setInt(4,tradAmbito.getIdAmbito());
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionAmbito.modificar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public void eliminar(String idIdioma, int idAmbito) throws Exception {
		PreparedStatement ps = null;
		String sql = "DELETE FROM traduccionesAmbitos WHERE idIdioma=? AND idAmbito=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,idIdioma);
			ps.setInt(2,idAmbito);
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionAmbito.eliminar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public Collection recuperarTodos() throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		TraduccionAmbitoDTO tradAmbito = null;
		String sql = "SELECT * FROM traduccionesAmbitos";
		
		try {
		    ps = con.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
				tradAmbito = new TraduccionAmbitoDTO();
				tradAmbito.setIdIdioma(rs.getString(1));
				tradAmbito.setIdAmbito(rs.getInt(2));
				tradAmbito.setNombre(rs.getString(3));
				tradAmbito.setDescripcion(rs.getString(4));
				c.add(tradAmbito);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionAmbito.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}
	
	public Collection recuperarPorIdAmbito(int idAmbito) throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		TraduccionAmbitoDTO tradAmbito = null;
		String sql = "SELECT * FROM traduccionesAmbitos WHERE idAmbito=?";
		
		try {
		    ps = con.prepareStatement(sql);
		    ps.setInt(1,idAmbito);
			rs = ps.executeQuery();
			while (rs.next()) {
			    tradAmbito = new TraduccionAmbitoDTO();
				tradAmbito.setIdIdioma(rs.getString(1));
				tradAmbito.setIdAmbito(rs.getInt(2));
				tradAmbito.setNombre(rs.getString(3));
				tradAmbito.setDescripcion(rs.getString(4));
				c.add(tradAmbito);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionAmbito.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}
	
	public Collection recuperarPorIdIdioma(String idIdioma) throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		TraduccionAmbitoDTO tradAmbito = null;
		String sql = "SELECT * FROM traduccionesAmbitos WHERE idIdioma=?";
		
		try {
		    ps = con.prepareStatement(sql);
		    ps.setString(1,idIdioma);
			rs = ps.executeQuery();
			while (rs.next()) {
			    tradAmbito = new TraduccionAmbitoDTO();
				tradAmbito.setIdIdioma(rs.getString(1));
				tradAmbito.setIdAmbito(rs.getInt(2));
				tradAmbito.setNombre(rs.getString(3));
				tradAmbito.setDescripcion(rs.getString(4));
				c.add(tradAmbito);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionAmbito.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}

}