/*
 * TraduccionAsociacionDAO.java		03-ago-2005
 */
package codetic.dao;

import java.sql.*;
import java.util.*;

import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que interactúa con la tabla TraduccionAsociaciones de la base de datos. 
 *
 * @author Miguel Angel Serna Mata
 */
public class TraduccionAsociacionDAO {
	
	private Connection con;
	
	public TraduccionAsociacionDAO(Connection con) {
		this.con = con;
	}
	
	public void crear(TraduccionAsociacionDTO tradAsociacion) throws DAOException{
	    if (recuperar(tradAsociacion.getIdIdioma(),tradAsociacion.getIdAsociacion()) != null) {
	     throw new DAOException("errors.dao.traduccionAsociacion.claveDuplicada");   
	    }
	    PreparedStatement ps = null;
		String sql = "INSERT INTO traduccionesAsociaciones VALUES (?,?,?,?,?)";
		
	    try{
	        ps = con.prepareStatement(sql);
			ps.setString(1,tradAsociacion.getIdIdioma());
			ps.setInt(2,tradAsociacion.getIdAsociacion());
			ps.setString(3,tradAsociacion.getLocalizacionGeografica());
			ps.setString(4,tradAsociacion.getDescripcion());
			ps.setString(5,tradAsociacion.getComentarios());
			ps.executeUpdate();
	    } catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionAsociacion.crear");
	    }
	    
	    JDBCUtils.close(ps);
	}
	
	public TraduccionAsociacionDTO recuperar(String idIdioma, int idAsociacion) {
		PreparedStatement ps = null;
		ResultSet rs = null;
		TraduccionAsociacionDTO tradAsociacion = null;
		String sql = "SELECT * FROM traduccionesAsociaciones WHERE idIdioma=? AND idAsociacion=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,idIdioma);
			ps.setInt(2,idAsociacion);
			rs = ps.executeQuery();   
			if (rs.next()) {
			    tradAsociacion = new TraduccionAsociacionDTO();
			    tradAsociacion.setIdIdioma(rs.getString(1));
			    tradAsociacion.setIdAsociacion(rs.getInt(2));
			    tradAsociacion.setLocalizacionGeografica(rs.getString(3));
			    tradAsociacion.setDescripcion(rs.getString(4));
			    tradAsociacion.setComentarios(rs.getString(5));
			}
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionAsociacion.recuperar");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return(tradAsociacion);
	}
	
	public void modificar(TraduccionAsociacionDTO tradAsociacion) throws Exception {
	    if (recuperar(tradAsociacion.getIdIdioma(),tradAsociacion.getIdAsociacion()) == null) {
		     throw new DAOException("errors.dao.traduccionAsociacion.noExiste");   
		    }
		PreparedStatement ps = null;
		String sql = "UPDATE traduccionesAsociaciones SET localizacionGeografica=?, descripcion=?, comentarios=?"+
		             "WHERE idIdioma=? AND idAsociacion=?";
		
		try {
		    ps = con.prepareStatement(sql);
		    ps.setString(1,tradAsociacion.getLocalizacionGeografica());
			ps.setString(2,tradAsociacion.getDescripcion());
			ps.setString(3,tradAsociacion.getComentarios());
			ps.setString(4,tradAsociacion.getIdIdioma());
			ps.setInt(5,tradAsociacion.getIdAsociacion());
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionAsociacion.modificar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public void eliminar(String idIdioma, int idAsociacion) throws Exception {
		PreparedStatement ps = null;
		String sql = "DELETE FROM traduccionesAsociaciones WHERE idIdioma=? AND idAsociacion=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,idIdioma);
			ps.setInt(2,idAsociacion);
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionAsociacion.eliminar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public Collection recuperarTodos() throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		TraduccionAsociacionDTO tradAsociacion = null;
		String sql = "SELECT * FROM traduccionesAsociaciones";
		
		try {
		    ps = con.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
			    tradAsociacion = new TraduccionAsociacionDTO();
			    tradAsociacion.setIdIdioma(rs.getString(1));
			    tradAsociacion.setIdAsociacion(rs.getInt(2));
			    tradAsociacion.setLocalizacionGeografica(rs.getString(3));
			    tradAsociacion.setDescripcion(rs.getString(4));
			    tradAsociacion.setComentarios(rs.getString(5));
				c.add(tradAsociacion);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionAsociacion.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}
	
	public Collection recuperarPorIdAsociacion(int idAsociacion) throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		TraduccionAsociacionDTO tradAsociacion = null;
		String sql = "SELECT * FROM traduccionesAsociaciones WHERE idAsociacion=?";
		
		try {
		    ps = con.prepareStatement(sql);
		    ps.setInt(1,idAsociacion);
			rs = ps.executeQuery();
			while (rs.next()) {
			    tradAsociacion = new TraduccionAsociacionDTO();
			    tradAsociacion.setIdIdioma(rs.getString(1));
			    tradAsociacion.setIdAsociacion(rs.getInt(2));
			    tradAsociacion.setLocalizacionGeografica(rs.getString(3));
			    tradAsociacion.setDescripcion(rs.getString(4));
			    tradAsociacion.setComentarios(rs.getString(5));
				c.add(tradAsociacion);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionAsociacion.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}

}


