/*
 * TraduccionCodigoDAO.java		03-ago-2005
 */
package codetic.dao;

import java.sql.*;
import java.util.*;

import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que interactúa con la tabla TraduccionesCodigos de la base de datos. 
 *
 * @author Miguel Angel Serna Mata
 */
public class TraduccionCodigoDAO {
	
	private Connection con;
	
	public TraduccionCodigoDAO(Connection con) {
		this.con = con;
	}
	
	public void crear(TraduccionCodigoDTO tradCodigo) throws DAOException{
	    if (recuperar(tradCodigo.getIdIdioma(),tradCodigo.getIdCodigo()) != null) {
	     throw new DAOException("errors.dao.traduccionCodigo.claveDuplicada");   
	    }
	    PreparedStatement ps = null;
		String sql = "INSERT INTO traduccionesCodigos VALUES (?,?,?,?,?,?,?,?)";
		
	    try{
	        ps = con.prepareStatement(sql);
			ps.setString(1,tradCodigo.getIdIdioma());
			ps.setInt(2,tradCodigo.getIdCodigo());
			ps.setString(3,tradCodigo.getTitulo());
			ps.setString(4,tradCodigo.getTextoCabecera());
			ps.setString(5,tradCodigo.getTextoPie());
			ps.setString(6,tradCodigo.getTraductores());
			ps.setString(7,tradCodigo.getComentarios());
			ps.setString(8,tradCodigo.getWebTraduccion());
			ps.executeUpdate();
	    } catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionCodigo.crear");
	    }
	    
	    JDBCUtils.close(ps);
	}
	
	public TraduccionCodigoDTO recuperar(String idIdioma, int idCodigo) {
		PreparedStatement ps = null;
		ResultSet rs = null;
		TraduccionCodigoDTO tradCodigo = null;
		String sql = "SELECT * FROM traduccionesCodigos WHERE idIdioma=? AND idCodigo=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,idIdioma);
			ps.setInt(2,idCodigo);
			rs = ps.executeQuery();   
			if (rs.next()) {
			    tradCodigo = new TraduccionCodigoDTO();
			    tradCodigo.setIdIdioma(rs.getString(1));
			    tradCodigo.setIdCodigo(rs.getInt(2));
			    tradCodigo.setTitulo(rs.getString(3));
			    tradCodigo.setTextoCabecera(rs.getString(4));
			    tradCodigo.setTextoPie(rs.getString(5));
			    tradCodigo.setTraductores(rs.getString(6));
			    tradCodigo.setComentarios(rs.getString(7));
			    tradCodigo.setWebTraduccion(rs.getString(8));
			}
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionCodigo.recuperar");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return(tradCodigo);
	}
	
	public void modificar(TraduccionCodigoDTO tradCodigo) throws Exception {
	    if (recuperar(tradCodigo.getIdIdioma(),tradCodigo.getIdCodigo()) == null) {
		     throw new DAOException("errors.dao.traduccionCodigo.noExiste");   
		    }
		PreparedStatement ps = null;
		String sql = "UPDATE traduccionesCodigos SET titulo=?, textoCabecera=?, textoPie=?, traductores=?, comentarios=?, webTraduccion=?"+
		             "WHERE idIdioma=? AND idCodigo=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,tradCodigo.getTitulo());
			ps.setString(2,tradCodigo.getTextoCabecera());
			ps.setString(3,tradCodigo.getTextoPie());
			ps.setString(4,tradCodigo.getTraductores());
			ps.setString(5,tradCodigo.getComentarios());
			ps.setString(6,tradCodigo.getWebTraduccion());
			ps.setString(7,tradCodigo.getIdIdioma());
			ps.setInt(8,tradCodigo.getIdCodigo());
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionCodigo.modificar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public void eliminar(String idIdioma, int idCodigo) throws Exception {
		PreparedStatement ps = null;
		String sql = "DELETE FROM traduccionesCodigos WHERE idIdioma=? AND idCodigo=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,idIdioma);
			ps.setInt(2,idCodigo);
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionCodigo.eliminar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public Collection recuperarTodos() throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		TraduccionCodigoDTO tradCodigo = null;
		String sql = "SELECT * FROM traduccionesCodigos";
		
		try {
		    ps = con.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
			    tradCodigo = new TraduccionCodigoDTO();
			    tradCodigo.setIdIdioma(rs.getString(1));
			    tradCodigo.setIdCodigo(rs.getInt(2));
			    tradCodigo.setTitulo(rs.getString(3));
			    tradCodigo.setTextoCabecera(rs.getString(4));
			    tradCodigo.setTextoPie(rs.getString(5));
			    tradCodigo.setTraductores(rs.getString(6));
			    tradCodigo.setComentarios(rs.getString(7));
			    tradCodigo.setWebTraduccion(rs.getString(8));
				c.add(tradCodigo);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionCodigo.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}
	
	public Collection recuperarPorIdCodigo(int idCodigo) throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		TraduccionCodigoDTO tradCodigo = null;
		String sql = "SELECT * FROM traduccionesCodigos WHERE idCodigo=?";
		
		try {
		    ps = con.prepareStatement(sql);
		    ps.setInt(1,idCodigo);
			rs = ps.executeQuery();
			while (rs.next()) {
			    tradCodigo = new TraduccionCodigoDTO();
			    tradCodigo.setIdIdioma(rs.getString(1));
			    tradCodigo.setIdCodigo(rs.getInt(2));
			    tradCodigo.setTitulo(rs.getString(3));
			    tradCodigo.setTextoCabecera(rs.getString(4));
			    tradCodigo.setTextoPie(rs.getString(5));
			    tradCodigo.setTraductores(rs.getString(6));
			    tradCodigo.setComentarios(rs.getString(7));
			    tradCodigo.setWebTraduccion(rs.getString(8));
				c.add(tradCodigo);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionCodigo.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}

}