/*
 * TraduccionGrupoDeTemasDAO.java		03-ago-2005
 */
package codetic.dao;

import java.sql.*;
import java.util.*;
import codetic.utils.*;

import codetic.dto.*;
import codetic.exceptions.*;

/**
 * Clase que interactúa con la tabla TraduccionesTemas de la base de datos. 
 *
 * @author Miguel Angel Serna Mata
 */
public class TraduccionGrupoDeTemasDAO {
	
	private Connection con;
	
	public TraduccionGrupoDeTemasDAO(Connection con) {
		this.con = con;
	}
	
	public void crear(TraduccionGrupoDeTemasDTO tradGrupoDeTemas) throws DAOException{
	    if (recuperar(tradGrupoDeTemas.getIdIdioma(),tradGrupoDeTemas.getIdGrupoDeTemas()) != null) {
	     throw new DAOException("errors.dao.traduccionAmbito.claveDuplicada");   
	    }
	    PreparedStatement ps = null;
		String sql = "INSERT INTO traduccionesGruposDeTemas VALUES (?,?,?,?)";
		
	    try{
	        ps = con.prepareStatement(sql);
			ps.setString(1,tradGrupoDeTemas.getIdIdioma());
			ps.setInt(2,tradGrupoDeTemas.getIdGrupoDeTemas());
			ps.setString(3,tradGrupoDeTemas.getNombre());
			ps.setString(4,tradGrupoDeTemas.getDescripcion());
			ps.executeUpdate();
	    } catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionAmbito.crear");
	    }
	    
	    JDBCUtils.close(ps);
	}
	
	public TraduccionGrupoDeTemasDTO recuperar(String idIdioma, int idGrupoDeTemas) {
		PreparedStatement ps = null;
		ResultSet rs = null;
		TraduccionGrupoDeTemasDTO tradGrupoDeTemas = null;
		String sql = "SELECT * FROM traduccionesGruposDeTemas WHERE idIdioma=? AND idGrupoDeTemas=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,idIdioma);
			ps.setInt(2,idGrupoDeTemas);
			rs = ps.executeQuery();   
			if (rs.next()) {
			    tradGrupoDeTemas = new TraduccionGrupoDeTemasDTO();
			    tradGrupoDeTemas.setIdIdioma(rs.getString(1));
			    tradGrupoDeTemas.setIdGrupoDeTemas(rs.getInt(2));
			    tradGrupoDeTemas.setNombre(rs.getString(3));
			    tradGrupoDeTemas.setDescripcion(rs.getString(4));
			}
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionAmbito.recuperar");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return(tradGrupoDeTemas);
	}
	
	public void modificar(TraduccionGrupoDeTemasDTO tradGrupoDeTemas) throws Exception {
	    if (recuperar(tradGrupoDeTemas.getIdIdioma(),tradGrupoDeTemas.getIdGrupoDeTemas()) == null) {
		     throw new DAOException("errors.dao.traduccionAmbito.noExiste");   
		    }
		PreparedStatement ps = null;
		String sql = "UPDATE traduccionesGruposDeTemas SET nombre=?, descripcion=?"+
		             "WHERE idIdioma=? AND idGrupoDeTemas=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,tradGrupoDeTemas.getNombre());
			ps.setString(2,tradGrupoDeTemas.getDescripcion());
			ps.setString(3,tradGrupoDeTemas.getIdIdioma());
			ps.setInt(4,tradGrupoDeTemas.getIdGrupoDeTemas());
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionAmbito.modificar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public void eliminar(String idIdioma, int idGrupoDeTemas) throws Exception {
		PreparedStatement ps = null;
		String sql = "DELETE FROM traduccionesGruposDeTemas WHERE idIdioma=? AND idGrupoDeTemas=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,idIdioma);
			ps.setInt(2,idGrupoDeTemas);
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionAmbito.eliminar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public Collection recuperarTodos() throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		TraduccionGrupoDeTemasDTO tradGrupoDeTemas = null;
		String sql = "SELECT * FROM traduccionesGruposDeTemas";
		
		try {
		    ps = con.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
				tradGrupoDeTemas = new TraduccionGrupoDeTemasDTO();
				tradGrupoDeTemas.setIdIdioma(rs.getString(1));
			    tradGrupoDeTemas.setIdGrupoDeTemas(rs.getInt(2));
			    tradGrupoDeTemas.setNombre(rs.getString(3));
			    tradGrupoDeTemas.setDescripcion(rs.getString(4));
				c.add(tradGrupoDeTemas);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionAmbito.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}
	
	public Collection recuperarPorIdGrupoDeTemas(int idGrupoDeTemas) throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		TraduccionGrupoDeTemasDTO tradGrupoDeTemas = null;
		String sql = "SELECT * FROM traduccionesGruposDeTemas WHERE idGrupoDeTemas=?";
		
		try {
		    ps = con.prepareStatement(sql);
		    ps.setInt(1,idGrupoDeTemas);
			rs = ps.executeQuery();
			while (rs.next()) {
			    tradGrupoDeTemas = new TraduccionGrupoDeTemasDTO();
				tradGrupoDeTemas.setIdIdioma(rs.getString(1));
				tradGrupoDeTemas.setIdGrupoDeTemas(rs.getInt(2));
				tradGrupoDeTemas.setNombre(rs.getString(3));
				tradGrupoDeTemas.setDescripcion(rs.getString(4));
				c.add(tradGrupoDeTemas);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionAmbito.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}
	
	public Collection recuperarPorIdIdioma(String idIdioma) throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		TraduccionGrupoDeTemasDTO tradGrupoDeTemas = null;
		String sql = "SELECT * FROM traduccionesGruposDeTemas WHERE idIdioma=?";
		
		try {
		    ps = con.prepareStatement(sql);
		    ps.setString(1,idIdioma);
			rs = ps.executeQuery();
			while (rs.next()) {
			    tradGrupoDeTemas = new TraduccionGrupoDeTemasDTO();
				tradGrupoDeTemas.setIdIdioma(rs.getString(1));
				tradGrupoDeTemas.setIdGrupoDeTemas(rs.getInt(2));
				tradGrupoDeTemas.setNombre(rs.getString(3));
				tradGrupoDeTemas.setDescripcion(rs.getString(4));
				c.add(tradGrupoDeTemas);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionAmbito.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}

}
