/*
 * TraduccionSeccionDAO.java		03-ago-2005
 */
package codetic.dao;

import java.sql.*;
import java.util.*;

import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que interactúa con la tabla TraduccionesSecciones de la base de datos. 
 *
 * @author Miguel Angel Serna Mata
 */
public class TraduccionSeccionDAO {
	
	private Connection con;
	
	public TraduccionSeccionDAO(Connection con) {
		this.con = con;
	}
	
	public void crear(TraduccionSeccionDTO tradSeccion) throws DAOException{
	    if (recuperar(tradSeccion.getIdIdioma(),tradSeccion.getIdCodigo(),tradSeccion.getNivel()) != null) {
	     throw new DAOException("errors.dao.traduccionSeccion.claveDuplicada");   
	    }
	    PreparedStatement ps = null;
		String sql = "INSERT INTO traduccionesSecciones VALUES (?,?,?,?,?,?)";
		
	    try{
	        ps = con.prepareStatement(sql);
			ps.setString(1,tradSeccion.getIdIdioma());
			ps.setInt(2,tradSeccion.getIdCodigo());
			ps.setString(3,tradSeccion.getNivel());
			ps.setString(4,tradSeccion.getTitulo());
			ps.setString(5,tradSeccion.getSeudotitulo());
			ps.setString(6,tradSeccion.getTexto());
			ps.executeUpdate();
	    } catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionSeccion.crear");
	    }
	    
	    JDBCUtils.close(ps);
	}
	
	public TraduccionSeccionDTO recuperar(String idIdioma, int idCodigo, String nivel) {
		PreparedStatement ps = null;
		ResultSet rs = null;
		TraduccionSeccionDTO tradSeccion = null;
		String sql = "SELECT * FROM traduccionesSecciones WHERE idIdioma=? AND idCodigo=? AND nivel=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,idIdioma);
			ps.setInt(2,idCodigo);
			ps.setString(3,nivel);
			rs = ps.executeQuery();   
			if (rs.next()) {
			    tradSeccion = new TraduccionSeccionDTO();
			    tradSeccion.setIdIdioma(rs.getString(1));
			    tradSeccion.setIdCodigo(rs.getInt(2));
			    tradSeccion.setNivel(rs.getString(3));
			    tradSeccion.setTitulo(rs.getString(4));
			    tradSeccion.setSeudotitulo(rs.getString(5));
			    tradSeccion.setTexto(rs.getString(6));
			}
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionSeccion.recuperar");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return(tradSeccion);
	}
	
	public void modificar(TraduccionSeccionDTO tradSeccion) throws Exception {
	    if (recuperar(tradSeccion.getIdIdioma(),tradSeccion.getIdCodigo(),tradSeccion.getNivel()) == null) {
		     throw new DAOException("errors.dao.traduccionSeccion.noExiste");   
		}
		PreparedStatement ps = null;
		String sql = "UPDATE traduccionesSecciones SET titulo=?, seudotitulo=?, texto=?"+
		             "WHERE idIdioma=? AND idCodigo=? AND nivel=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,tradSeccion.getTitulo());
			ps.setString(2,tradSeccion.getSeudotitulo());
			ps.setString(3,tradSeccion.getTexto());
			ps.setString(4,tradSeccion.getIdIdioma());
			ps.setInt(5,tradSeccion.getIdCodigo());
		    ps.setString(6,tradSeccion.getNivel());
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionSeccion.modificar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public void eliminar(String idIdioma, int idCodigo, String nivel) throws Exception {
		PreparedStatement ps = null;
		String sql = "DELETE FROM traduccionesSecciones WHERE idIdioma=? AND idCodigo=? AND nivel=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,idIdioma);
			ps.setInt(2,idCodigo);
			ps.setString(3,nivel);
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionSeccion.eliminar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public void eliminarPorIdIdiomaYIdCodigo(String idIdioma, int idCodigo) throws Exception {
		PreparedStatement ps = null;
		String sql = "DELETE FROM traduccionesSecciones WHERE idIdioma=? AND idCodigo=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,idIdioma);
			ps.setInt(2,idCodigo);
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionSeccion.eliminar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public Collection recuperarTodos() throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		TraduccionSeccionDTO tradSeccion = null;
		String sql = "SELECT * FROM traduccionesSecciones";
		
		try {
		    ps = con.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
			    tradSeccion = new TraduccionSeccionDTO();
			    tradSeccion.setIdIdioma(rs.getString(1));
			    tradSeccion.setIdCodigo(rs.getInt(2));
			    tradSeccion.setNivel(rs.getString(3));
			    tradSeccion.setTitulo(rs.getString(4));
			    tradSeccion.setSeudotitulo(rs.getString(5));
			    tradSeccion.setTexto(rs.getString(6));
				c.add(tradSeccion);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionSeccion.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}
	
	public Collection recuperarPorIdIdiomaYIdCodigo(String idIdioma, int idCodigo) throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		TraduccionSeccionDTO tradSeccion = null;
		String sql = "SELECT * FROM traduccionesSecciones WHERE idIdioma=? AND idCodigo=?";
		
		try {
		    ps = con.prepareStatement(sql);
		    ps.setString(1,idIdioma);
		    ps.setInt(2,idCodigo);
			rs = ps.executeQuery();
			while (rs.next()) {
			    tradSeccion = new TraduccionSeccionDTO();
			    tradSeccion.setIdIdioma(rs.getString(1));
			    tradSeccion.setIdCodigo(rs.getInt(2));
			    tradSeccion.setNivel(rs.getString(3));
			    tradSeccion.setTitulo(rs.getString(4));
			    tradSeccion.setSeudotitulo(rs.getString(5));
			    tradSeccion.setTexto(rs.getString(6));
				c.add(tradSeccion);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionSeccion.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}
	
	public Collection recuperarPorIdCodigoYIdNivel(int idCodigo, String nivel) throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		TraduccionSeccionDTO tradSeccion = null;
		String sql = "SELECT * FROM traduccionesSecciones WHERE idCodigo=? AND nivel=?";
		
		try {
		    ps = con.prepareStatement(sql);
		    ps.setInt(1,idCodigo);
		    ps.setString(2,nivel);
			rs = ps.executeQuery();
			while (rs.next()) {
			    tradSeccion = new TraduccionSeccionDTO();
			    tradSeccion.setIdIdioma(rs.getString(1));
			    tradSeccion.setIdCodigo(rs.getInt(2));
			    tradSeccion.setNivel(rs.getString(3));
			    tradSeccion.setTitulo(rs.getString(4));
			    tradSeccion.setSeudotitulo(rs.getString(5));
			    tradSeccion.setTexto(rs.getString(6));
				c.add(tradSeccion);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionSeccion.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}
	
	public Collection recuperarPorIdIdiomaYIdCodigoDelIndice(String idIdioma, int idCodigo) throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		TraduccionSeccionDTO tradSeccion = null;
		String sql = "SELECT * FROM traduccionesSecciones WHERE idIdioma=? AND idCodigo=? AND titulo != '' AND LOCATE('.',nivel) = 0 ORDER BY nivel";
		
		try {
		    ps = con.prepareStatement(sql);
		    ps.setString(1,idIdioma);
		    ps.setInt(2,idCodigo);
			rs = ps.executeQuery();
			while (rs.next()) {
			    tradSeccion = new TraduccionSeccionDTO();
			    tradSeccion.setIdIdioma(rs.getString(1));
			    tradSeccion.setIdCodigo(rs.getInt(2));
			    tradSeccion.setNivel(rs.getString(3));
			    tradSeccion.setTitulo(rs.getString(4));
			    tradSeccion.setSeudotitulo(rs.getString(5));
			    tradSeccion.setTexto(rs.getString(6));
				c.add(tradSeccion);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionSeccion.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}

}