/*
 * TraduccionTemaDAO.java		03-ago-2005
 */
package codetic.dao;

import java.sql.*;
import java.util.*;

import codetic.dto.*;
import codetic.exceptions.*;
import codetic.utils.*;

/**
 * Clase que interactúa con la tabla TraduccionesTemas de la base de datos. 
 *
 * @author Miguel Angel Serna Mata
 */
public class TraduccionTemaDAO {
	
	private Connection con;
	
	public TraduccionTemaDAO(Connection con) {
		this.con = con;
	}
	
	public void crear(TraduccionTemaDTO tradTema) throws DAOException{
	    if (recuperar(tradTema.getIdIdioma(),tradTema.getIdTema()) != null) {
	     throw new DAOException("errors.dao.traduccionTema.claveDuplicada");   
	    }
	    PreparedStatement ps = null;
		String sql = "INSERT INTO traduccionesTemas VALUES (?,?,?,?,?)";
		
	    try{
	        ps = con.prepareStatement(sql);
			ps.setString(1,tradTema.getIdIdioma());
			ps.setInt(2,tradTema.getIdTema());
			ps.setString(3,tradTema.getNombre());
			ps.setString(4,tradTema.getDescripcion());
			ps.setString(5,tradTema.getPalabrasClave());
			ps.executeUpdate();
	    } catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionTema.crear");
	    }
	    
	    JDBCUtils.close(ps);
	}
	
	public TraduccionTemaDTO recuperar(String idIdioma, int idTema) {
		PreparedStatement ps = null;
		ResultSet rs = null;
		TraduccionTemaDTO tradTema = null;
		String sql = "SELECT * FROM traduccionesTemas WHERE idIdioma=? AND idTema=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,idIdioma);
			ps.setInt(2,idTema);
			rs = ps.executeQuery();   
			if (rs.next()) {
			    tradTema = new TraduccionTemaDTO();
			    tradTema.setIdIdioma(rs.getString(1));
			    tradTema.setIdTema(rs.getInt(2));
			    tradTema.setNombre(rs.getString(3));
			    tradTema.setDescripcion(rs.getString(4));
			    tradTema.setPalabrasClave(rs.getString(5));
			}
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionTema.recuperar");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return(tradTema);
	}
	
	public void modificar(TraduccionTemaDTO tradTema) throws Exception {
	    if (recuperar(tradTema.getIdIdioma(),tradTema.getIdTema()) == null) {
		     throw new DAOException("errors.dao.traduccionTema.noExiste");   
		    }
		PreparedStatement ps = null;
		String sql = "UPDATE traduccionesTemas SET nombre=?, descripcion=?, palabrasClave=?"+
		             "WHERE idIdioma=? AND idTema=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,tradTema.getNombre());
			ps.setString(2,tradTema.getDescripcion());
			ps.setString(3,tradTema.getPalabrasClave());
			ps.setString(4,tradTema.getIdIdioma());
			ps.setInt(5,tradTema.getIdTema());
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionTema.modificar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public void eliminar(String idIdioma, int idTema) throws Exception {
		PreparedStatement ps = null;
		String sql = "DELETE FROM traduccionesTemas WHERE idIdioma=? AND idTema=?";
		
		try {
		    ps = con.prepareStatement(sql);
			ps.setString(1,idIdioma);
			ps.setInt(2,idTema);
			ps.executeUpdate();    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionTema.eliminar");
	    }
		
		JDBCUtils.close(ps);
	}
	
	public Collection recuperarTodos() throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		TraduccionTemaDTO tradTema = null;
		String sql = "SELECT * FROM traduccionesTemas";
		
		try {
		    ps = con.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
				tradTema = new TraduccionTemaDTO();
				tradTema.setIdIdioma(rs.getString(1));
			    tradTema.setIdTema(rs.getInt(2));
			    tradTema.setNombre(rs.getString(3));
			    tradTema.setDescripcion(rs.getString(4));
			    tradTema.setPalabrasClave(rs.getString(5));
				c.add(tradTema);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionTema.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}
	
	public Collection recuperarPorIdTema(int idTema) throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		TraduccionTemaDTO tradTema = null;
		String sql = "SELECT * FROM traduccionesTemas WHERE idTema=?";
		
		try {
		    ps = con.prepareStatement(sql);
		    ps.setInt(1,idTema);
			rs = ps.executeQuery();
			while (rs.next()) {
			    tradTema = new TraduccionTemaDTO();
				tradTema.setIdIdioma(rs.getString(1));
				tradTema.setIdTema(rs.getInt(2));
				tradTema.setNombre(rs.getString(3));
				tradTema.setDescripcion(rs.getString(4));
				tradTema.setPalabrasClave(rs.getString(5));
				c.add(tradTema);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException("errors.dao.traduccionTema.recuperarTodos");
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}
	
	public Collection recuperarPorIdGrupoDeTemasYIdIdioma(int idGrupoDeTemas, String idIdioma) throws Exception {
		PreparedStatement ps = null;
		ResultSet rs = null;
		Collection c = new ArrayList();
		TraduccionTemaDTO tradTema = null;
		String sql = "SELECT * FROM traduccionesTemas tt, temas t, gruposDeTemas g WHERE g.idGrupoDeTemas = t.idGrupoDeTemas" +
				" AND t.idTema = tt.idTema AND g.idGrupoDeTemas=? AND tt.idIdioma=?";
		
		try {
		    ps = con.prepareStatement(sql);
		    ps.setInt(1,idGrupoDeTemas);
		    ps.setString(2,idIdioma);
			rs = ps.executeQuery();
			while (rs.next()) {
			    tradTema = new TraduccionTemaDTO();
				tradTema.setIdIdioma(rs.getString(1));
				tradTema.setIdTema(rs.getInt(2));
				tradTema.setNombre(rs.getString(3));
				tradTema.setDescripcion(rs.getString(4));
				tradTema.setPalabrasClave(rs.getString(5));
				c.add(tradTema);
			}    
		} catch (SQLException e) {
	        throw new RuntimeException(e.getMessage());/*"errors.dao.traduccionTema.recuperarTodos");*/
	    }
		
		JDBCUtils.close(rs);
		JDBCUtils.close(ps);
		
		return c;
	}

}
